/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file global.h
 *   @brief This is the global header (like an extern.h).
 *
 *   It contains all global variables (and Extern functions).
 */
/*  ************************************************************************************************  */

#ifndef GLOBAL_H
#define GLOBAL_H

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <string.h>

#ifndef _WIN32
  #include <sys/un.h>
#endif

#include "sysconf.h"
#include "datatypes.h"

#ifdef USE_MYSQL
  #ifdef MYSQL_HEADER_PREFIX
    #include <mysql/mysql.h>
  #else
    #include <mysql.h>
  #endif
#endif

#ifdef _WIN32
  #ifdef MODULE_COMPILE
    #define E __declspec( dllimport )
  #else
    #define E extern __declspec( dllexport )
  #endif
#else
  #define E extern
#endif

E ns_conf ns_conf_l, *ns;
E cs_conf cs_conf_l, *cs;
E hs_conf hs_conf_l, *hs;
E ms_conf ms_conf_l, *ms;
E bs_conf bs_conf_l, *bs;
E g_conf g_conf_l, *g;

E Q_conf Q_conf_l, *Q;
E O_conf O_conf_l, *O;
E sockscontrol_conf sc_conf_l, *SC;
E config_load copyconfig, *m;

/* CServe.c */
E Threads 	*threads;
E Threads 	*IRCThread;
E Users   	*users;
E int	 	IRCSock;
// - Protocol
E Protocol IRC;
E Protocol WebForm;
// - Q
E char Q_Avail_Chanmodes[];
E char Q_Avail_Usermodes[];
E char L_Avail_Usermodes[];
E char encoding[];

/******** Global variables! ********/
E int logchan;                /* -logchan */
E int logging;                /* -logging (alog();)*/
E int sqllogging;             /* -sqllogging (sqllog();)*/
E int readonly;               /* -services do not accept user queries */
E int dofork;                 /* -fork */
E int noexpire;
E int debugbylog;

/** LOG FILES **/
E FILE *quotelog;
E FILE *InputLog;
/***************/

E time_t ExpireCheckTime;
E time_t sqldumptime;

E time_t sockstartcheck;
E time_t sockendtime;
E time_t defcon_last_flood;
E time_t socktimediff;

E int32 socks_autodefcon_status;
E int32 socks_remember_defcon;
E int32 sockstatus;
E int32 sockemergency; /* if you set that parameter to 1, sockscontrol will gline all clients that are connecting too fast.
        				WARNING: if you set it to 1 before launching orion, sockscontrol will gline all users on the network by launching
        				orion, because orion gets all client connects VERY fast by connecting and that's fast enough that it's over the limit
        				so sockscontrol will act like you told him to act by a connectionflood.
        			*/
E int32 defcon; //don't change that digit until you launched orion.
E bool noop; //Don't set it to 1 before starting orion!! - that will kill all ircoperators.
E bool DuringServerlink;
E bool netinfo;
E bool connected;

E char linkserver		[MAX_SERVER_LEN];
E char squitmsg			[MAX_QUITMSG_LEN];
E char global_target	[MAX_MESSAGE_LEN];
E char testedDebugLog	[300];

E LevelChanging leveltable[MAX_CHANNEL_LEVELS];

#ifdef USE_THREADS
E pthread_mutex_t limit_lock;
E pthread_mutex_t dlog_lock;
E pthread_mutex_t sqllog_lock;
E pthread_mutex_t alog_lock;
E pthread_mutex_t mail_lock;
E pthread_mutex_t modules_read_lock;
E pthread_mutex_t modules_lock;
E pthread_mutex_t mysql_lock;
E pthread_mutex_t quotelog_lock;
E pthread_mutex_t mysqlbuffer_lock;
E pthread_mutex_t users_lock;
E pthread_mutex_t protocol_lock;
E pthread_mutex_t backtrace_lock;
E pthread_mutex_t stays_lock;
E pthread_mutex_t substring_lock;
E pthread_mutex_t trusts_lock;
E pthread_mutex_t trustgroups_lock;
E pthread_mutex_t ulines_lock;
E pthread_mutex_t servers_lock;
E pthread_mutex_t thread_lock;
E pthread_mutex_t globvar_lock;
E pthread_mutex_t shconfig_lock;
E pthread_mutex_t languages_lock;

E pthread_mutex_t sockets_lock;
E pthread_mutex_t sockread_lock;
E pthread_mutex_t sockwrite_lock;
E pthread_mutex_t threads_lock;
#endif

#ifndef USE_THREADS
E fd_set master;
E fd_set read_fds;
E struct sockaddr_in serveraddr;
E struct sockaddr_in clientaddr;
#endif

/* servers.c */
E Servers *servlist;	/* This is a list of all Servers 	*/
E Servers *me_server;	/* This are we        				*/
E Servers *serv_uplink;	/* This is our uplink 				*/
E char uplink[MAX_SERVER_LEN];
E char *TS6UPLINK;
E char *TS6SID;

/* modules.c */
E void* languages[NUM_LANGS];
E char modul[20][MAX_NICK_LEN];	/* 20 modules | each MAX_NICK_LEN chars long */
E SharedLanguage *ModuleLang[NUM_LANGS];

/* base64.c */
E char *base64enc(long i);
E long base64dec(char *b64);
E long base64dects(char *ts);
E int b64_encode(char *src, size_t srclength, char *target, size_t targsize);
E int b64_decode(char *src, char *target, size_t targsize);
E char *encode_ip(unsigned char *ip);
E int decode_ip(char *buf);

/* encrypt.c */
E int newest_encryption;

/* mysql.c */
/* Database Global Variables */
E MYSQL mysql;              /* MySQL Handler */
E MYSQL_FIELD *fields;      /* MySQL Fields  */
E MYSQL_RES *socks_res;     /* MySQL Result  */
E MYSQL_ROW socks_row;      /* MySQL Row     */
E MySQLBuf MySQLSave[MAX_BUFFER_TABLES];	// For Collecting queries on mass connects

/* Webform */
#ifndef _WIN32
E PROTOCOL_BOOL orion_WRCtranslate_cmd_initialize(Threads *thread);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_add_account(Threads *thread, char *uid, char *authnick);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_del_account(Threads *thread, char *uid);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_svskill(Threads *thread, char *source, char *user, char *buf);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_kill(Threads *thread, char *source, char *user, char *buf);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_svsmode(Threads *thread, char *nick, char *target, char *modes);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_svs2mode(Threads *thread, char *nick, char *target, char *modes);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_notice(Threads *thread, char *source, char *dest, char *buf);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_quit(Threads *thread, char *source, char *buf);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_protoctl(Threads *thread);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_pass(Threads *thread, char *pass);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_server(Threads *thread, char *servname, int hop, char *descript);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_pong(Threads *thread, char *servname, char *who);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_ping(Threads *thread, char *servname, char *who);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_join(Threads *thread, char *user, char *channel, time_t chantime);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_invite(Threads *thread, char *source, char *chan, char *nick);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_part(Threads *thread, char *nick, char *chan, char *buf);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_wallops(Threads *thread, char *source, char *fmt);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_global(Threads *thread, char *source, char *buf);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_squit(Threads *thread, char *servname, char *message);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_svso(Threads *thread, char *source, char *nick, char *flag);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_connect(Threads *thread, bool how);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_svshold(Threads *thread, char *nick);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_release_svshold(Threads *thread, char *nick);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_unban(Threads *thread, char *name, char *nick);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_svspart(Threads *thread, char *source, char *nick, char *chan);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_sapart(Threads *thread, char *source, char *nick, char *chan);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_swhois(Threads *thread, char *source, char *who, char *mask);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_eob(Threads *thread);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason);
E PROTOCOL_BOOL orion_WRCtranslate_cmd_global_legacy(Threads *thread, char *source, char *fmt);

E PROTOCOL_BOOL orion_WRCtranslate_cmd_raw(Threads *thread, char *param[]);
E bool orion_WRCtranslate_valid_chan(Threads *thread, char *chan);
E bool orion_WRCtranslate_valid_nick(Threads *thread, char *nick);
E bool orion_WRCtranslate_flood_mode_check(Threads *thread, char *value);
#endif
#endif
