/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file osocket.h
 *   @brief This is the socket header.
 *
 *   It contains all functions which are used to handle sockets and the IRCd Protocol
 */
/*  ************************************************************************************************  */

#ifndef OSOCKET_H
#define OSOCKET_H

#include "datatypes.h"

#ifdef _WIN32
  #ifdef MODULE_COMPILE
    #define DLL __declspec( dllimport )
  #else
    #define DLL __declspec( dllexport )
  #endif
#else
  #define DLL extern
#endif

#ifdef _WIN32
  DLL char *orion_sockstrerror(int error);
  DLL int orion_socksetnonb(orion_socket_t fd);
#endif

DLL int32 sputs(char *str, Threads *thread);
int32 sread(Threads *thread, char *buf, int len);
char *sgets2(char *buf, int len, Threads *thread);
char *sgets(char *buf, int len, Threads *thread);
int32 sungetc(int c, Threads *thread);
int32 sgetc(Threads *thread);

DLL void main_raw(Threads *thread, const char *fmt, ...);
DLL void web_raw(Threads *thread, const char *fmt, ...);
DLL void buildAndSendWebformMessage(Threads *thread);
int32 startlink(Threads *thread, const char *host, int port, const char *lhost, int lport);

void startservices(Threads *thread);
DLL void exitservices(Threads *thread);
DLL void closelink(Threads *thread);

/** Commands.c **/
DLL PROTOCOL_BOOL do_privmsg			(Threads *thread, char *nick, char *target, const char *message, ...);
DLL PROTOCOL_BOOL do_global        		(Threads *thread, char *snick, const char *message, ...);
DLL PROTOCOL_BOOL do_wallops			(Threads *thread, char *snick, const char *message, ...);
DLL PROTOCOL_BOOL do_del_account		(Threads *thread, char *uid);
DLL PROTOCOL_BOOL do_add_account		(Threads *thread, char *uid, char *authnick);
DLL PROTOCOL_BOOL do_modes_on_identify	(Threads *thread, Users *user, char *authnick);
DLL PROTOCOL_BOOL do_modes_on_logout	(Threads *thread, Users *user);
DLL PROTOCOL_BOOL do_invite        		(Threads *thread, char *snick, char *nick, char *chan);
DLL PROTOCOL_BOOL do_server        		(Threads *thread, char *servername, const char *serverdescription, ...);
DLL PROTOCOL_BOOL do_kickban       		(Threads *thread, char *snick, char *channel, time_t date, int bantype, Condata *Cdata, const char *reason, ...);
DLL PROTOCOL_BOOL do_logchan       		(Threads *thread, char *snick, char *nick, char *ident, char *rhost, const char *message, ...);
DLL PROTOCOL_BOOL do_server_notice 		(Threads *thread, const char *message, ...);
DLL PROTOCOL_BOOL createuser       		(Threads *thread, int fake, char *in_nick, char *ident, char *host, char *modes, char *realname);


/** Gline.c **/
DLL PROTOCOL_BOOL do_gline			(Threads *thread, char *servername, char *mask, char *nick, time_t in_expiretime, time_t in_hubtime, char *in_reason);
DLL PROTOCOL_BOOL do_ungline		(Threads *thread, char *servername, char *mask, char *nick, time_t in_hubtime);
DLL PROTOCOL_BOOL on_gline			(Threads *thread, char *servername, char *mask, char *nick, time_t in_expiretime, time_t in_hubtime, char *in_reason);
DLL PROTOCOL_BOOL on_ungline		(Threads *thread, char *servername, char *mask);


/** Join.c **/
DLL PROTOCOL_BOOL do_sajoin			(Threads *thread, int fake, char *snick, char *nick, char *chan, time_t date);
DLL PROTOCOL_BOOL do_svsjoin		(Threads *thread, int fake, char *snick, char *nick, char *chan, time_t date);
DLL PROTOCOL_BOOL do_services_join	(Threads *thread, char *snick, char *channel, time_t time_in, int fake);
DLL PROTOCOL_BOOL do_join			(Threads *thread, int fake, char *nick, char *channel, time_t date);
DLL PROTOCOL_BOOL _join				(Threads *thread, int fake, char *nick, char *chanbuf, time_t time_in);


/** Kick.c **/
DLL PROTOCOL_BOOL do_BSkick			(Threads *thread, char *nick, char *channel, char *knick, const char *reason, ...);
DLL PROTOCOL_BOOL do_kick			(Threads *thread, char *nick, char *channel, char *knick, const char *reason, ...);
DLL PROTOCOL_BOOL do_kick_nopart	(Threads *thread, char *nick, char *channel, char *knick, const char *reason, ...);
DLL PROTOCOL_BOOL on_kick        	(Threads *thread, char *sender ,char *nick, char *channel);
DLL PROTOCOL_BOOL kick           	(Threads *thread, char *nick, char *channel);


/** Kill.c **/
DLL PROTOCOL_BOOL do_svskill		(Threads *thread, char *nick, char *target, const char *message, ...);
DLL PROTOCOL_BOOL do_kill			(Threads *thread, char *nick, char *target, const char *message, ...);
DLL PROTOCOL_BOOL _kill				(Threads *thread, char *nick);


/** Link.c **/
DLL PROTOCOL_BOOL on_linkdone 		(Threads *thread);
DLL PROTOCOL_BOOL on_link     		(Threads *thread, char *LinkedServer, char *Uplink);


/** Mode.c **/
DLL PROTOCOL_BOOL do_mode     		(Threads *thread, char *nick, char *target, time_t date, const char *buf, ...);
DLL PROTOCOL_BOOL on_mode     		(Threads *thread, char *nick, char *target, time_t date, char *buf);
DLL PROTOCOL_BOOL do_svs2mode 		(Threads *thread, char *nick, char *target, const char *modes, ...);
DLL PROTOCOL_BOOL do_svsmode  		(Threads *thread, char *nick, char *target, const char *modes, ...);
DLL PROTOCOL_BOOL svsmode     		(Threads *thread, char *nick, char *target, char *modes);


/** Modes.c **/
DLL PROTOCOL_BOOL do_rawmode		(Threads *thread, char *nick, char *target, time_t time_in, char *what, ...);
DLL PROTOCOL_BOOL do_raw2mode		(Threads *thread, char *source, char *channel, char *modes, char *targetnick, time_t time_in);
DLL PROTOCOL_BOOL do_rawkill		(Threads *thread, char *nick, char *target, const char *message, ...);
DLL PROTOCOL_BOOL do_modecompress	(Threads *thread, char *source, char *channel, char *modes, char *targetnick, time_t time_in);
DLL PROTOCOL_BOOL do_rawsvsmode		(Threads *thread, char *source, char *nick, char *modes);
DLL PROTOCOL_BOOL do_rawtopic		(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when);


/** Nick.c **/
DLL PROTOCOL_BOOL do_nick    		(Threads *thread, char *oldnick, char *newnick);
DLL PROTOCOL_BOOL do_svsnick 		(Threads *thread, char *snick, char *oldnick, char *newnick);
DLL PROTOCOL_BOOL chgnick    		(Threads *thread, char *oldnick, char *newnick);
DLL int cnick						(Threads *thread, char *nick, char *vident, char *ident, char *modes, time_t time_in, char *rhost, char *vhost, char *server, char *realname, char *ip, int fake);


/** Notice.c **/
DLL PROTOCOL_BOOL do_notice      	(Threads *thread, char *nick, char *target, char *message, ...);
DLL PROTOCOL_BOOL do_notice_lang 	(Threads *thread, void *lang, char *nick, char *target, char *message, ...);


/** Part.c **/
DLL PROTOCOL_BOOL do_sapart  		(Threads *thread, char *snick, char *nick, char *chan);
DLL PROTOCOL_BOOL do_svspart 		(Threads *thread, char *snick, char *nick, char *chan);
DLL PROTOCOL_BOOL do_part    		(Threads *thread, char *nick, char *chan, char *partmsg);
DLL PROTOCOL_BOOL on_part    		(Threads *thread, char *user, char *in_channel);
DLL PROTOCOL_BOOL part       		(Threads *thread, char *user, char *in_channel);
DLL PROTOCOL_BOOL partall			(Threads *thread, char *nick);


/** Protocol.c **/
DLL PROTOCOL_BOOL on_sethost 		(Threads *thread, char *nick,char *vhost);
DLL PROTOCOL_BOOL on_chghost 		(Threads *thread, char *nick, char *vhost);
DLL PROTOCOL_BOOL on_setident		(Threads *thread, char *nick, char *ident);
DLL PROTOCOL_BOOL on_chgident		(Threads *thread, char *nick, char *ident);
DLL PROTOCOL_BOOL do_chghost 		(Threads *thread, char *snick, char *nick, char *host);
DLL PROTOCOL_BOOL do_chgident		(Threads *thread, char *snick, char *nick, char *ident);
DLL PROTOCOL_BOOL on_chgname 		(Threads *thread, char *nick, char *name);
DLL PROTOCOL_BOOL version    		(Threads *thread, char *nick);
DLL PROTOCOL_BOOL on_privmsg 		(Threads *thread, char *param[]);


/** Qline.c **/
DLL PROTOCOL_BOOL do_sqline			(Threads *thread, char *snick, char *nickmask, time_t in_expiretime, time_t in_hubtime, const char *reason, ...);
DLL PROTOCOL_BOOL on_sqline			(Threads *thread, char *snick, char *nickmask, time_t in_expiretime, time_t in_hubtime, char *in_reason);
DLL PROTOCOL_BOOL do_unsqline		(Threads *thread, char *snick, char *servername, char *nickmask);
DLL PROTOCOL_BOOL on_unsqline		(Threads *thread, char *snick, char *servername, char *nickmask);


/** Quit.c **/
DLL PROTOCOL_BOOL do_quit  			(Threads *thread, char *snick, char *message, ...);
DLL PROTOCOL_BOOL on_quit  			(Threads *thread, char *nick);
DLL PROTOCOL_BOOL quit     			(Threads *thread, char *nick);

DLL PROTOCOL_BOOL do_squit 			(Threads *thread, char *QServer, char *message, ...);
DLL PROTOCOL_BOOL on_squit 			(Threads *thread, char *QServer);
DLL PROTOCOL_BOOL squit    			(Threads *thread, char *QServer);


/** Send.c **/
DLL PROTOCOL_BOOL msend     		(Threads *thread, bool howtosend, char *nick, char *target, const char *message, ...);
DLL PROTOCOL_BOOL send_lang 		(Threads *thread, void *lang, bool howtosend, char *nick, char *target, const char *message, ...);
DLL PROTOCOL_BOOL moduleSendLang	(Threads *thread, int language, bool howtosend, char *nick, char *target, char *message, ...);
DLL PROTOCOL_BOOL vsend_cmd 		(Threads *thread, const char *source, const char *fmt, va_list args);
DLL PROTOCOL_BOOL send_cmd  		(Threads *thread, const char *source, const char *fmt, ...);


/** Sjoin.c **/
DLL PROTOCOL_BOOL sjoin				(Threads *thread, char *channel, time_t date, char *buf, int fake);


/** Slink.c **/
DLL PROTOCOL_BOOL on_serviceslink		(Threads *thread);
DLL PROTOCOL_BOOL on_serviceslink_done 	(Threads *thread);


/** Socket.c **/
DLL void irc_check_command 			(Threads *thread, char *receive_in);
DLL void web_check_command			(Threads *thread, char *receive_in);


/** Stats.c **/
DLL PROTOCOL_BOOL on_stats			(Threads *thread, char *who, char *user, char *what);


/** Topic.c **/
DLL PROTOCOL_BOOL do_topic 			(Threads *thread, char *nick, char *last_setter, char *channel, time_t date, const char *ftopic, ...);
DLL PROTOCOL_BOOL topic    			(Threads *thread, char *nick, char *channel, time_t date, char *topic);
DLL PROTOCOL_BOOL on_topic 			(Threads *thread, char *nick, char *channel, time_t date, char *ftopic);


/** Zline.c **/
DLL PROTOCOL_BOOL do_szline			(Threads *thread, char *servername, char *ipmask, char *nick, time_t in_expiretime, time_t in_hubtime, char *in_reason);
DLL PROTOCOL_BOOL do_unszline		(Threads *thread, char *servername, char *ipmask, char *nick, time_t in_hubtime);
DLL PROTOCOL_BOOL on_szline			(Threads *thread, char *servername, char *ipmask, char *nick, time_t in_expiretime, time_t in_hubtime, char *in_reason);
DLL PROTOCOL_BOOL on_unszline		(Threads *thread, char *servername, char *ipmask);
#endif
