/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file commands.c
 *   @brief This is the Module file for COMMANDS (Server Side) Commands.
 *
 *   It contains all functions which are neccessary send
 *   COMMANDS from Orion.
 *
 *   The COMMANDS File is used to send Actions to the Uplink
 *   (Main functions for all cliente and the especially the server.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "osocket.h"
#include "webform.h"

/**
 * @brief do_privmsg:  Sends out a PRIVMSG (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The nick (services-nick) to send the message
 * @param target The target (CHANNEL/NICK) where we want to put the information
 * @param message The message we want to send
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_privmsg(Threads *thread, char *nick, char *target, const char* message, ...){
	/* Declare variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */
	bzero(&tmp,sizeof(tmp));
	
	/* Let's check out inputs - are they valid? */
	if(nick==NULL || target==NULL || message==NULL){
		return CMD_ERROR;
	}
	
	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);
	
	return orion_WRCtranslate_cmd_privmsg(thread,nick,target,tmp);
}

/*  ************************************************************************************************  */

/**
 * @brief do_kickban:  Sends out a KICKBAN (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The nick (services-nick) to send the kickban
 * @param channel The channel where we want to kick and ban the USer
 * @param date The channel-creation-date (has to provided with MODE)
 * @param bantype The Bantype for target channel
 * @param Cdata The ConnectionData struct which holds the Users Host,Ident, .. (user who we like to ban)
 * @param reason The Reason why we kickban that user in target channel
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_kickban(Threads *thread, char *snick, char *channel, time_t date, int bantype, Condata *Cdata, const char *reason, ...){
	/* Declare variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise variables to (NULL) */
	bzero(&tmp,sizeof(tmp));

	/* Let's check out inputs - are they valid? */
	if(snick==NULL || channel==NULL || reason==NULL || Cdata==NULL)
		return CMD_ERROR;

	/* Join reason string */
	va_start	(va,  reason);
	vsnprintf	(tmp, sizeof(tmp)-1, reason, va);
	va_end		(va);

	/* Analyse Command - is it supported from the IRCd !? */
	if(channel[0]=='#'){
        if(GetUser(Cdata->nick)==NULL)
	    	return CMD_ERROR;

		do_mode(thread, snick, channel, date == 0 ? time(NULL) : date, "+%c %s",IRC.BanMode,BuildBanMask(thread,Cdata,bantype));
		do_kick(thread, snick, channel, Cdata->nick, tmp);
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief do_global:  Sends out a GLOBOPS (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The nick (services-nick) to send the kickban
 * @param message The message we want to send
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_global(Threads *thread, char *snick, const char *message, ...){
	/* Declare variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise variables to (NULL) */
	bzero(&tmp,sizeof(tmp));
	
	/* Let's check out inputs - are they valid? */
	if(snick==NULL || message==NULL)
		return CMD_ERROR;

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);

	return orion_WRCtranslate_cmd_global(thread,snick,tmp);
}

/*  ************************************************************************************************  */

/**
 * @brief do_wallops:  Sends out a WALLOPS (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The nick (services-nick) to send the kickban
 * @param message The message we want to send
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_wallops(Threads *thread, char *snick, const char *message, ...){
	/* Declare variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise variables to (NULL) */
	bzero(&tmp,sizeof(tmp));
	
	/* Let's check out inputs - are they valid? */
	if(snick==NULL || message==NULL)
		return CMD_ERROR;
	
	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);
	
	return orion_WRCtranslate_cmd_wallops(thread,snick,tmp);
}

/*  ************************************************************************************************  */

/**
 * @brief do_del_account:  Sends out a del Account (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param uid The users uid (target for which we delete the accout)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_del_account(Threads *thread, char *uid){
	/* Let's check out inputs - are they valid? */
	if(uid==NULL)
		return CMD_ERROR;
	
	return orion_WRCtranslate_cmd_del_account(thread,uid);
}

/*  ************************************************************************************************  */

/**
 * @brief do_add_account:  Sends out an add Account (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param uid The users uid (target for which we add the accout)
 * @param authnick The users identification nick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_add_account(Threads *thread, char *uid, char *authnick){
	/* Let's check out inputs - are they valid? */
	if(uid==NULL || authnick==NULL)
		return CMD_ERROR;
	
	return orion_WRCtranslate_cmd_add_account(thread,uid,authnick);
}

/*  ************************************************************************************************  */

/**
 * @brief do_modes_on_identify:  Sends out right modes, account information on identify
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param user The User object.
 * @param authnick The users authnick
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_modes_on_identify(Threads *thread, Users *user, char *authnick){
	/* Let's check out inputs - are they valid? */
	if(user==NULL || authnick==NULL)
		return CMD_ERROR;
	
	if(IRC.p10new){
		do_add_account(thread,user->nick,authnick);
	}
	
	if(IRC.NickRegMode){
		do_svs2mode(thread,ns->nick,user->nick,IRC.ModeOnNickReg);
	}
	
	if(IRC.p10protocol){
		do_add_account(thread,user->uid,authnick);
	}
	
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief do_modes_on_logout:  Sends out right modes, account information on logout
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param user The User object.
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_modes_on_logout(Threads *thread, Users *user){
	/* Let's check out inputs - are they valid? */
	if(user==NULL)
		return CMD_ERROR;
					
	if(IRC.p10protocol)
		do_del_account(thread,user->uid);
	
	if(IRC.p10new)
		do_del_account(thread,user->nick);
	
	if(IRC.NickRegMode)
		do_svs2mode(thread,ns->nick,user->nick,IRC.ModeOnNickDrop);
	
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief do_logchan:  Sends out a PRIVMSG (depending on the Loaded Protocol) - and writes it to the logfile
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The nick (services-nick) to send the kickban
 * @param nick The nick which used the command we want to log
 * @param ident The Ident of that user (to provide a more detailed Information to the OPs)
 * @param rhost The realhost(IP) of that user (to provide a more detailed Information to the OPs)
 * @param message The message we want to send (only to inform Ops and logging)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_logchan(Threads *thread, char *snick, char *nick, char *ident, char *rhost, const char *message, ...){
	/* Deklarate variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise variables to (NULL) */
	bzero(&tmp,sizeof(tmp));
	
	/* Let's check out inputs - are they valid? */
	if(message==NULL)
		return CMD_ERROR;

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);

	/* Analyze Command - is it supported from the IRCd !? */
	if((ident == NULL || rhost == NULL) && nick == NULL && snick == NULL && logchan){
		do_privmsg(thread,g->nick,m->logchan,_dynadd("%s",tmp));
		if(logging) alog(thread,"%s",tmp);
		return CMD_EXECUTED;
	}
	else if((ident == NULL || rhost == NULL) && nick == NULL && logchan){
		do_privmsg(thread,g->nick,m->logchan,_dynadd("%s: %s",snick,tmp));
		if(logging) alog(thread,"%s: %s",snick,tmp);
		return CMD_EXECUTED;
	}	
	else if((ident == NULL || rhost == NULL) && logchan){
		do_privmsg(thread,g->nick,m->logchan,_dynadd("%s: %s: %s",snick,nick,tmp));
		if(logging) alog(thread,"%s: %s: %s",snick,nick,tmp);
		return CMD_EXECUTED;
	}
	else if(logchan){
		do_privmsg(thread,g->nick,m->logchan,_dynadd("%s: %s!%s@%s %s",snick,nick,ident,rhost,tmp));
		if(logging) alog(thread,"%s: %s!%s@%s %s",snick,nick,ident,rhost,tmp);
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief do_invite:  Sends out an INVITE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The nick (services-nick) to send the kickban
 * @param nick The nick which we want to invite
 * @param chan The Channel where we want to invite him to
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_invite(Threads *thread, char *snick, char *nick, char *chan){
	/* Let's check out inputs - are they valid? */
	if(snick==NULL || nick==NULL || chan==NULL)
		return CMD_ERROR;
	
	return orion_WRCtranslate_cmd_invite(thread, snick, chan, nick);
}

/*  ************************************************************************************************  */

/**
 * @brief do_server:  Sends out a SERVER (depending on the Loaded Protocol) - and writes it to the logfile
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName for the new server
 * @param serverdescription The ServerDescription for the new server
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_server(Threads *thread, char *servername, const char *serverdescription, ...){
	/* Declare variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise variables to (NULL) */
	bzero(&tmp,sizeof(tmp));
	
	/* Let's check out inputs - are they valid? */
	if(servername==NULL || serverdescription==NULL)
		return CMD_ERROR;
	
	/* Join reason string */
	va_start	(va, serverdescription);
	vsnprintf	(tmp, sizeof(tmp)-1, serverdescription, va);
	va_end		(va);
	
	return orion_WRCtranslate_cmd_jupe(thread,servername,"services",tmp);
}

/*  ************************************************************************************************  */

/**
 * @brief do_server_notice:  Sends out a NOTICE (depending on the Loaded Protocol) - through all servers
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param message The message we want to send (SERVER NOTICE - provided through all server)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_server_notice(Threads *thread, const char *message, ...){
	/* Declare variables */
	va_list va;
	char tmp[BUFFER+1];
	PROTOCOL_BOOL analyzer;
	
	/* Initialise variables to (NULL) */
	bzero(&tmp,sizeof(tmp));
	
	/* Let's check out inputs - are they valid? */
	if(message==NULL)
		return CMD_ERROR;
	
	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);

#ifdef USE_THREADS
	my_pthread_mutex_lock(&globvar_lock);
#endif
	if(g->loaded)	analyzer = orion_WRCtranslate_cmd_serv_notice(thread, g->nick, global_target, tmp);
	else			analyzer = orion_WRCtranslate_cmd_serv_notice(thread, m->servername, global_target, tmp);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&globvar_lock);
#endif
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief createuser:  Sends out a NICK (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param fake The faketype (to tell the program it's a nick that was created by itself) usually >0
 * @param in_nick The nick which we would like to use
 * @param ident The Ident for created user
 * @param host The realhost(IP) of the created user
 * @param modes The modes to connect with
 * @param realname The Realname for the created User
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL createuser(Threads *thread, int fake, char *in_nick ,char *ident, char *host, char *modes, char *realname){
	/* Declare variables */
	PROTOCOL_BOOL analyzer;
	
	/* Let's check out inputs - are they valid? */
	if(in_nick==NULL || ident==NULL || host==NULL || modes==NULL || realname==NULL)
		return CMD_ERROR;
	
	/* Analyse Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_nick(thread, in_nick, ident, host, realname, modes);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;
	
	cnick(thread, in_nick, NULL, ident, modes, time(NULL), host, host, m->servername, realname, "127.0.0.1", fake);
	return analyzer;
}

/*  ************************************************************************************************  */
