/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 370                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-26 17:17:04 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file config.c
 *   @brief This is the Module file for loading/proofing the configfiles.
 *
 *   It contains all functions which are neccessary to
 *   handle the Config-Files.
 *
 *   Reads the configfiles and saves the data into the struct's (Q->nick)
 */
/*  ************************************************************************************************  */

#include "sysconf.h"
#include "datatypes.h"
#include "global.h"
#include "modules.h"
#include "services.h"

/**
 * @brief proof_config:  Proofes the Config (Does an Error-Check)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int 0 - We had an Error, 1 - Config seems to be in good shape
 **/
bool proof_config(Threads *thread){
	
	if(copyconfig.server[0] == '\0'){
		alog(thread,"You forgot to define \"Server\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
    }

	if(copyconfig.port < 0){
		alog(thread,"You forgot to define \"Port\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.WebPort < 0){
		alog(thread,"You forgot to define \"WebPort\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.WebFormIp[0] == '\0'){
		alog(thread,"You forgot to define \"WebFormIp\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.linkpass[0] == '\0'){
		alog(thread,"You forgot to define \"LinkPass\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.WebSocketPass[0] == '\0' && copyconfig.UseWebForm){
		alog(thread,"You forgot to define \"WebSocketPass\", this is REQUIRED (you wanted to use WebForm - check \"UseWebForm\")!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.serverlink < 0){
		alog(thread,"You forgot to define \"ServerLink\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.servername[0] == '\0'){
		alog(thread,"You forgot to define \"ServerName\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.description[0] == '\0'){
		alog(thread,"You forgot to define \"Description\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	/*if(copyconfig.modes[0] == '\0'){
		alog(thread,"You forgot to define \"Modes\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}*/

	if(copyconfig.host[0] == '\0'){
		alog(thread,"You forgot to define \"ServicesHost\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.nick[0] == '\0'){
		alog(thread,"You forgot to define \"Nick\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.ident[0] == '\0'){
		alog(thread,"You forgot to define \"Ident\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.vhost[0] == '\0'){
		alog(thread,"You forgot to define \"Vhost\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.homepage[0] == '\0'){
		alog(thread,"You forgot to define \"Homepage\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.protocol[0] == '\0'){
		alog(thread,"You forgot to define \"Protocol\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.pidfilename[0] == '\0'){
		alog(thread,"You forgot to define \"PidFilename\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.LoadModulesConf[0] == '\0'){
		alog(thread,"You forgot to define \"LoadModulesConf\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.ReadTimeout < 0){
		alog(thread,"You forgot to define \"ReadTimeout\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.ReadTimeout_us < 0){
		alog(thread,"You forgot to define \"ReadTimeout_us\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.KeepWhowasLog < 0 && copyconfig.UseWhowas == 1){
		alog(thread,"You forgot to define \"KeepWhowasLog\", this is REQUIRED!, if you use UseWhowas, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.GlobalOnCycleMessage[0] == '\0' && copyconfig.GlobalOnCycle){
		alog(thread,"You forgot to define \"GlobalOnCycleMessage\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.sendmailpath[0] == '\0'){
		alog(thread,"You forgot to define \"sendmailpath\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.SendFrom[0] == '\0'){
		alog(thread,"You forgot to define \"SendFrom\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    		
	if(copyconfig.language < 0){
		alog(thread,"You forgot to define \"NSDefLanguage\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
    if(copyconfig.UseMail == -1 && copyconfig.NSEmailReg==1){
		alog(thread,"You forgot to define \"UseMail\", this is REQUIRED (denpency on NSEmailReg)!, define it - Config Error\n");
    	return 0;
	}
    	
    if(copyconfig.NSRExpire < 0 && copyconfig.NSEmailReg==1){
		alog(thread,"You forgot to define \"NSRExpire\", this is REQUIRED (denpency on NSEmailReg)!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.NSExpire[0] == '\0'){
		alog(thread,"You forgot to define \"NSExpire\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    
	if(copyconfig.NSMaxGroups < 0){
		alog(thread,"You forgot to define \"NSMaxGroups\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    
	if(copyconfig.NSMaxAliases < 0){
		alog(thread,"You forgot to define \"NSMaxAliases\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
		   	
	if(copyconfig.NSAccessMax < 0){
		alog(thread,"You forgot to define \"NSAccessMax\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.NSReleaseTimeout < 0){
		alog(thread,"You forgot to define \"NSReleaseTimeout\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.NSListMax < 0){
		alog(thread,"You forgot to define \"NSListMax\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
		
	if(copyconfig.NSGuestNickPrefix[0] == '\0'){
		alog(thread,"You forgot to define \"NSGuestNickPrefix\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.CSExpire[0] == '\0'){
		alog(thread,"You forgot to define \"CSExpire\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.CSDefBantype < 0){
		alog(thread,"You forgot to define \"CSDefBantype\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}else if(copyconfig.CSDefBantype > 3){
		alog(thread,"ERROR \"CSDefBantype\" cannot be higher then 3\n");
    	return 0;
	}
	
	if(copyconfig.CSStayChannel < 0){
		alog(thread,"You forgot to define \"CSStayChannel\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.CSAccessMax < 0){
		alog(thread,"You forgot to define \"CSAccessMax\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.CSAutokickMax < 0){
		alog(thread,"You forgot to define \"CSAutokickMax\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.CSAutokickReason[0] == '\0'){
		alog(thread,"You forgot to define \"CSAutokickReason\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.CSListMax < 0){
		alog(thread,"You forgot to define \"CSListMax\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.BSMinUsers < 0){
		alog(thread,"You forgot to define \"BSMinUsers\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}

	if(copyconfig.BSBadWordsMax < 0){
		alog(thread,"You forgot to define \"BSBadWordsMax\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    
	if(copyconfig.HSWaitRequest[0] == '\0'){
		alog(thread,"You forgot to define \"HSWaitRequest\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.socklinetime < 0){
		alog(thread,"You forgot to define \"socklinetime\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.glineonclearchan[0] == '\0'){
		alog(thread,"You forgot to define \"GlineOnClearchan\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.GlineExpiry < 0){
		alog(thread,"You forgot to define \"GlineExpiry\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.maxconnections < 0){
		alog(thread,"You forgot to define \"DefSessionLimit\", this is REQUIRED!, define it - Config Error\n");
		return 0;
	}
	
	if(copyconfig.OSMsgFloodProt == 1 && (!copyconfig.OSMsgFloodlines  || !copyconfig.OSMsgFloodtime)){
		alog(thread,"You defines \"OSMsgFloodProt\" but you didn't define \"OSMsgFloodlines\" or \"OSMsgFloodtime\", check your Config! - Config Error\n");
    	return 0;
	}
	
	if(copyconfig.OSNickFloodProt == 1 && (!copyconfig.OSNickFloodlines || !copyconfig.OSNickFloodtime)){
		alog(thread,"You defines \"OSMsgFloodProt\" but you didn't define \"OSMsgFloodlines\" or \"OSMsgFloodtime\", check your Config! - Config Error\n");
    	return 0;
	}

	if(copyconfig.DefConKillReason[0] == '\0'){
		alog(thread,"You forgot to define \"DefConKillReason\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.mysqlsrv[0] == '\0'){
		alog(thread,"You forgot to define \"MysqlSrv\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.mysqluser[0] == '\0'){
		alog(thread,"You forgot to define \"MysqlUser\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.mysqlpass[0] == '\0'){
		alog(thread,"You forgot to define \"MysqlPass\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.mysqlDB[0] == '\0'){
		alog(thread,"You forgot to define \"MysqlLDB\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    	
	if(copyconfig.mysqlport < 0){
		alog(thread,"You forgot to define \"MysqlPort\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}
    
	if(copyconfig.encrypt[0] == '\0'){
		alog(thread,"You forgot to define \"Encryption\", this is REQUIRED!, define it - Config Error\n");
    	return 0;
	}	
    return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief zero_config_values:  Sets all Config-variables to their defaults
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void zero_config_values(Threads *thread){

    bzero(&copyconfig.server               ,sizeof(copyconfig.server));                //
    bzero(&copyconfig.bindServicesIp       ,sizeof(copyconfig.bindServicesIp));        //
    bzero(&copyconfig.WebFormIp            ,sizeof(copyconfig.WebFormIp));             //
    bzero(&copyconfig.bindWebFormIp        ,sizeof(copyconfig.bindWebFormIp));         //
    bzero(&copyconfig.linkpass             ,sizeof(copyconfig.linkpass));              //
    bzero(&copyconfig.WebSocketPass        ,sizeof(copyconfig.WebSocketPass));         //
    bzero(&copyconfig.servername           ,sizeof(copyconfig.servername));            //
    bzero(&copyconfig.vhost                ,sizeof(copyconfig.vhost));                 //
    bzero(&copyconfig.description          ,sizeof(copyconfig.description));           //
    bzero(&copyconfig.modes                ,sizeof(copyconfig.modes));                 //
    bzero(&copyconfig.host                 ,sizeof(copyconfig.host));                  //
    bzero(&copyconfig.nick                 ,sizeof(copyconfig.nick));                  //
    bzero(&copyconfig.ident                ,sizeof(copyconfig.ident));                 //
	bzero(&copyconfig.homepage             ,sizeof(copyconfig.homepage));              //
	bzero(&copyconfig.LinkToWebform        ,sizeof(copyconfig.LinkToWebform));         //
	bzero(&copyconfig.protocol             ,sizeof(copyconfig.protocol));              //
	bzero(&copyconfig.pidfilename          ,sizeof(copyconfig.pidfilename));           //
    bzero(&copyconfig.logfile              ,sizeof(copyconfig.logfile));
	bzero(&copyconfig.sqllogfile           ,sizeof(copyconfig.sqllogfile));
    bzero(&copyconfig.debuglog             ,sizeof(copyconfig.debuglog));
    bzero(&copyconfig.logquotes            ,sizeof(copyconfig.logquotes));
    bzero(&copyconfig.logchan              ,sizeof(copyconfig.logchan));
    bzero(&copyconfig.network              ,sizeof(copyconfig.network));               //
    bzero(&copyconfig.GlobalOnCycleMessage ,sizeof(copyconfig.GlobalOnCycleMessage));  // - if GlobalOnCycle == 1
    bzero(&copyconfig.GlobalOnCycleUp      ,sizeof(copyconfig.GlobalOnCycleUp));
    bzero(&copyconfig.sendmailpath         ,sizeof(copyconfig.sendmailpath));          //
    bzero(&copyconfig.SendFrom             ,sizeof(copyconfig.SendFrom));              //
	bzero(&copyconfig.NSGuestNickPrefix    ,sizeof(copyconfig.NSGuestNickPrefix));     //
    bzero(&copyconfig.CSAutokickReason     ,sizeof(copyconfig.CSAutokickReason));      //
    bzero(&copyconfig.BSFantasyCharacter   ,sizeof(copyconfig.BSFantasyCharacter));
	bzero(&copyconfig.glineonclearchan     ,sizeof(copyconfig.glineonclearchan));      //
	bzero(&copyconfig.SessionLimitExceeded ,sizeof(copyconfig.SessionLimitExceeded));
	bzero(&copyconfig.mysqlsrv             ,sizeof(copyconfig.mysqlsrv));              //
    bzero(&copyconfig.mysqluser            ,sizeof(copyconfig.mysqluser));             //
    bzero(&copyconfig.mysqlpass            ,sizeof(copyconfig.mysqlpass));             //
    bzero(&copyconfig.mysqlDB              ,sizeof(copyconfig.mysqlDB));               //
    bzero(&copyconfig.mysqlsocket          ,sizeof(copyconfig.mysqlsocket));           
    bzero(&copyconfig.DefConChanModes      ,sizeof(copyconfig.DefConChanModes));       //
	bzero(&copyconfig.DefconMessage        ,sizeof(copyconfig.DefconMessage));
	bzero(&copyconfig.DefConOffMessage     ,sizeof(copyconfig.DefConOffMessage));
	bzero(&copyconfig.DefConKillReason     ,sizeof(copyconfig.DefConKillReason));      //
	bzero(&copyconfig.NSExpire             ,sizeof(copyconfig.NSExpire));              //
	bzero(&copyconfig.CSExpire             ,sizeof(copyconfig.CSExpire));              //
	bzero(&copyconfig.HSWaitRequest        ,sizeof(copyconfig.HSWaitRequest));         //
	bzero(&copyconfig.MysqlMove            ,sizeof(copyconfig.MysqlMove));
	bzero(&copyconfig.NewEncrypt           ,sizeof(copyconfig.NewEncrypt));			   //	
	bzero(&copyconfig.encrypt              ,sizeof(copyconfig.encrypt));			   //	
	bzero(&copyconfig.LoadModulesConf      ,sizeof(copyconfig.LoadModulesConf));	   //
	
    copyconfig.starttime               = 0;
    copyconfig.port                    = -1; //
    copyconfig.WebPort                 = -1; //
    copyconfig.serverlink              = -1; //
	
    copyconfig.freelogs                = 0;
    copyconfig.KeepLogs                = expire("7d");
    copyconfig.debuglogging            = 0;
	copyconfig.debug                   = 0;
	copyconfig.AutoRecognizeModules    = 0;

    copyconfig.StrictPasswords         = 0;
    copyconfig.BadPassLimit            = 0;
    copyconfig.SecureIdentify          = 0;
	copyconfig.ReadTimeout             = -1; //
   	copyconfig.ReadTimeout_us          = -1; //
	copyconfig.UseWebForm              = 0;
	copyconfig.UseWhowas               = 0;
	copyconfig.KeepWhowasLog           = -1;
	copyconfig.LimitCheck              = expire("90s");
	copyconfig.TrustDumpTime           = expire("30m");
	copyconfig.ExpireCheckTime         = expire("10m");
	copyconfig.NickUsedCheck           = expire("20m");
   	copyconfig.ForceForbidReason       = 0;
   	copyconfig.UsePrivmsg              = 0;
	copyconfig.UseStrictPrivMsg        = 0;
    copyconfig.GlobalOnCycle           = 0;

    copyconfig.AnonymousGlobal         = 0;
    copyconfig.UseMail                 = 0;
    copyconfig.RestrictMail            = 0;
    copyconfig.DontQuoteAddresses      = 0;
    	
	copyconfig.NSDefKill               = 0;
	copyconfig.NSDefKillQuick          = 0;
	copyconfig.NSDefKillImmed          = 0;
    copyconfig.NSDefSecure             = 0;
	copyconfig.NSDefPrivate            = 0;
    copyconfig.NSDefHideEmail          = 0;
    copyconfig.NSDefHideUsermask       = 0;
    copyconfig.NSDefHideQuit           = 0;
    copyconfig.NSDefHideStatus         = 0;
    copyconfig.NSDefMsg                = 0;
    copyconfig.NSDefAutoop             = 0;
    copyconfig.NSDefIncognito          = 0;
    copyconfig.NSDefMemoSignon         = 0;
    copyconfig.NSDefMemoReceive        = 0;
    		
	copyconfig.language                = -1; //
	copyconfig.NSEmailReg			   = 0;
	copyconfig.NSRegDelay			   = expire("20s");
	copyconfig.NSResendDelay		   = expire("60m");
	copyconfig.NSRExpire               = -1;
	copyconfig.NSWaitRegister          = expire("30s");
	copyconfig.NSQuickKill             = -1;
	copyconfig.NSImmedKill             = -1;
	copyconfig.NSMaxGroups			   = -1;
    copyconfig.NSMaxAliases            = -1;
	copyconfig.NSAccessMax             = -1; //
	copyconfig.NSReleaseTimeout        = -1; //
    copyconfig.NSNoGroupChange         = 0;
    copyconfig.NSListOpersOnly         = 0;
    copyconfig.NSListMax               = -1; //
	copyconfig.NSSecureAdmins          = 0;
	copyconfig.NSRestrictGetPass       = 0;
	copyconfig.NSUseTempLinks          = 0;
    	
	copyconfig.CSDefNone               = 0;
    copyconfig.CSDefKeepTopic          = 0;
	copyconfig.CSDefOpNotice           = 0;
    copyconfig.CSDefPeace              = 0;
	copyconfig.CSDefPrivate            = 0;
    copyconfig.CSDefRestricted         = 0;
	copyconfig.CSDefSecure             = 0;
	copyconfig.CSDefSecureOps          = 0;
    copyconfig.CSDefSecureFounder      = 0;
	copyconfig.CSDefSignKick           = 0;
    copyconfig.CSDefSignKickLevel      = 0;
	copyconfig.CSDefTopicLock          = 0;
	copyconfig.CSDefXOP                = 0;
	copyconfig.CSDefInhabit            = 0;
	
	copyconfig.CSMaxReg                = -1;
	copyconfig.CSWaitRegister          = expire("30s");
    copyconfig.CSDefBantype            = -1; //
    copyconfig.CSStayChannel		   = -1; //
	copyconfig.CSAccessMax             = -1; //
    copyconfig.CSAutokickMax           = -1; //
    copyconfig.CSAutokickForceReason   = 0;

    copyconfig.CSListOpersOnly         = 0;
	copyconfig.CSListMax               = -1; //
    copyconfig.CSOpersOnly             = 0;

    copyconfig.maxmemos                = -1;
    copyconfig.MSUsePriorities         = 0;

	copyconfig.BSDefDontKickOps        = 0;
	copyconfig.BSDefDontKickVoices     = 0;
	copyconfig.BSDefGreet              = 0;
	copyconfig.BSDefFantasy            = 0;
    copyconfig.BSDefSymbiosis          = 0;
    copyconfig.BSMinUsers              = -1; //
	copyconfig.BSBadWordsMax           = -1; //
    copyconfig.BSGentleBWReason        = 0;
    copyconfig.BSCaseSensitive         = 0;

    copyconfig.socklinetime            = -1; //
    copyconfig.GlineExpiry             = -1; //
    	
	copyconfig.OSForceTrusts           = 0;
	copyconfig.OSGlineOnTrustViolation = 0;
    copyconfig.OSForceTrustDelete      = 0;
    copyconfig.OSUseRNlineIDline       = 0;
    
    copyconfig.OSMsgFloodKill		   = 0;
    
    copyconfig.OSMsgFloodProt          = 0;
    copyconfig.OSMsgFloodlines		   = 0;
    copyconfig.OSMsgFloodtime		   = 0;
    
    copyconfig.OSNickFloodProt		   = 0;
    copyconfig.OSNickFloodlines		   = 0;
    copyconfig.OSNickFloodtime 		   = 0;
    
	copyconfig.DisableRaw              = 0;
	
	copyconfig.WallBadOS               = 0;
	copyconfig.WallOSGlobal            = 0;
	copyconfig.WallOSMode              = 0;
	copyconfig.WallOSClearchan         = 0;
	copyconfig.WallOSKick              = 0;
	copyconfig.WallOSGline             = 0;
	copyconfig.WallOSNoOp              = 0;
	copyconfig.WallOSJupe              = 0;
	copyconfig.WallOSRaw               = 0;
	copyconfig.WallGetpass             = 0;
	copyconfig.WallSetpass             = 0;
	copyconfig.WallForbid              = 0;
	copyconfig.WallDrop                = 0;
	
	copyconfig.maxconnections          = -1; //
    copyconfig.MaxSessionKill          = -1;/*Not now required */
    copyconfig.SessionGlineExpiry      = expire("30m");

	copyconfig.OSOpersOnly             = 0;
	copyconfig.GlobalOnDefcon          = 0;
	
    copyconfig.mysqlport               = -1; //
    copyconfig.MySQLSuperUser		   = 0;

    /*### Traffic Settings ###*/

    copyconfig.TrafficOut              = 0;
    copyconfig.TrafficIn               = 0;
    copyconfig.TrafficIn2_bps          = 0;
    copyconfig.TrafficOut_bps          = 0;
    copyconfig.TrafficIn_bps           = 0;
    copyconfig.TrafficOut2_bps         = 0;

	delAllUlines(thread);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief loadconf:  Loads the ConfigFile for target Service
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modul The Services which's Config we want to load
 * @return void
 **/
void loadconf(Threads *thread, char *modul){ //
	/* Declare Variables */
	FILE *in;
	char path[MAX_FILEPATH_LEN];
	char line[1000];
	char desc[1000];
	char value[1000];
	char *array[4];

	/* Initialise Variables to (NULL) */
	bzero(&path,sizeof(path));
	
	snprintf(path,MAX_FILEPATH_LEN,"%s/",SERVICES_CONF_DIR);
	dynncat(path,modul,dynlen(modul));
	dynncat(path,".conf",5);
	
	dlog(thread,3,"loadconf(); path: %s\n",path);
	
	in = fopen(path,"r");
	
	if(in==NULL){
		printf("ERROR: Configfile not found !! - Check the path to your services.conf\n");
		printf("Tried path was \"%s\"\n",path);
		exit(0);
	}	
	
	bzero(&line,sizeof(line));
	while(fgets(line,sizeof(line)-1,in)){
		if(line[0]!='#' && line[1]!='\n'){
			bzero(&desc,sizeof(desc));
			bzero(&value,sizeof(value));
			bzero(array,sizeof(array));
			
			strclean(line);
			
			if(strcin('\"',line)){
				/*strsplit(line,array,'\"');
				
				array[1][dynlen(array[1])]='\0';
				dynncpy(value,array[1],sizeof(value));
				
				dynncpy(desc,array[0],sizeof(desc));
				strclean(desc);
				desc[dynlen(desc)-1]='\0';*/
				sscanf(line,"%s \"%[^\"]s",desc,value);
				//value[dynlen(value)-1]='\0';
				//printf("DESC: %s, VALUE: %s\n",desc,value);
			}else{
				sscanf(line,"%s %s",desc,value);
				strclean(value);
				strclean(desc);
			}
			
			bzero(line,sizeof(line));
			if(streqnc(modul, "services")) 
				main_build(thread,desc,value);
		}
		else if(streq(line,"#EOF")) break;
	}
	fclose(in);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief dumpconf_mysql:  Dumps the configuration file to the DB
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void dumpconf_mysql(Threads *thread){ //
	/* Declare Variables */	
	FILE *in;
	char path[MAX_FILEPATH_LEN];
	char line[1000];
	char desc[1000];
	char value[1000];
	char *array[4];

	/* Initialise Variables to (NULL) */
	bzero(path,sizeof(path));
	
	/* make path */
	snprintf(path,MAX_FILEPATH_LEN,"%s/",SERVICES_CONF_DIR);
	dynncat(path,"services",dynlen("services"));
	dynncat(path,".conf",5);
	
	dlog(thread,3,"dumpconf_mysql(); path: %s\n",path);
	
	in = fopen(path,"r");
	
	if(in==NULL){
		printf("ERROR: Configfile not found !! - Check the path to your services.conf\n");
		printf("Tried path was \"%s\"\n",path);
		exit(0);
	}	
	
	bzero(line,sizeof(line));
	while(fgets(line,sizeof(line)-1,in)){
		if(line[0]!='#' && line[1]!='\n'){
			bzero(desc,sizeof(desc));
			bzero(value,sizeof(value));
			bzero(array,sizeof(array));
			strclean(line);
			
			if(strcin('\"',line)){
				strsplit(line,array,'\"');
				
				array[1][dynlen(array[1])]='\0';
				dynncpy(value,array[1],sizeof(value));
				
				dynncpy(desc,array[0],sizeof(desc));
				strclean(desc);
				desc[dynlen(desc)-1]='\0';
				strdelc(desc,'\t');
			}else{
				sscanf(line,"%s %s",desc,value);
				strclean(value);
				strclean(desc);
			}
			
			bzero(line,sizeof(line));
			domysql(thread,"INSERT INTO `orion_core_config_ro` (`variable`,`value`) VALUES ('%s','%s')",mysql_quote(thread,desc),mysql_quote(thread,value));
		}
		else if(streq(line,"#EOF")) break;
	}
	fclose(in);
	mysql_free_quotes(thread);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief update_config:  Updates the main ConfigFile
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modul The Services which's Config we want to load
 * @return void
 **/
void update_config(Threads *thread, char *modul){ //
	/* Declare Variables */
	FILE *in;
	char path[MAX_FILEPATH_LEN];
	char line	[1000];
	char myline	[1000];
	char desc	[1000];
	char value	[1000];
	char newencryption[30];
	char *array[4];

	/* Initialise Variables to (NULL) */
	bzero(&path,sizeof(path));
	
	/* make path */
	snprintf(path,MAX_FILEPATH_LEN,"%s/",SERVICES_CONF_DIR);
	dynncat(path,modul,dynlen(modul));
	dynncat(path,".conf",5);
	
	dlog(thread,3,"update_config(); path: %s\n",path);
	
	in = fopen(path,"r");
	
	if(in==NULL){
		printf("ERROR: Configfile not found !! - Check the path to your services.conf\n");
		printf("Tried path was \"%s\"\n",path);
		exit(0);
	}	
	
	bzero(&line,sizeof(line));
	bzero(&newencryption,sizeof(newencryption));
	while(fgets(line,sizeof(line)-1,in)){
		if(line[0]!='#' && line[1]!='\n'){
			bzero(&myline,sizeof(myline));
			bzero(&desc,sizeof(desc));
			bzero(&value,sizeof(value));
			bzero(array,sizeof(array));
			
			dynncpy(myline, line, sizeof(myline));
			strclean(line);
			
			if(strcin('\"',line)){
				sscanf(line,"%s \"%[^\"]s",desc,value);
			}else{
				sscanf(line,"%s %s",desc,value);
				strclean(value);
				strclean(desc);
			}
			
			if(streqnc(desc, "NewEncryption")){
				fputs(_dynadd("#%s",myline), in);
				dynncpy(newencryption, value, sizeof(newencryption));
				continue;
			}
			
			if(streqnc(desc, "Encryption")){
				fputs(_dynadd("%s \"%s\"",desc, value), in);
				
				dynncpy(value, newencryption, sizeof(value));
			}
			
			bzero(line,sizeof(line));
			if(streqnc(modul, "services")) main_build(thread,desc,value);
		}
		else if(streq(line,"#EOF")) break;
	}
	fclose(in);
	//exit(0);
	return;
}	

/*  ************************************************************************************************  */

/**
 * @brief main_build:  Builds the MainConfig-Vars (puts the Values from the File to the Vars)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param desc The Description of the Config Setting (lvalue)
 * @param value The Value to which we should set the lvalue (rvalue)
 * @return void
 **/
void main_build(Threads *thread,char *desc,char *value){ //
		
	if(streqnc(desc, "server")){
    	dynncpy(copyconfig.server,value,sizeof(copyconfig.server));
    	return;
	}
	
	if(streqnc(desc, "BindServicesIP")){
    	dynncpy(copyconfig.bindServicesIp,value,sizeof(copyconfig.bindServicesIp));
    	return;
	}
    	
	if(streqnc(desc, "port")){
    	copyconfig.port=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "webformip")){
    	dynncpy(copyconfig.WebFormIp,value,sizeof(copyconfig.WebFormIp));
    	return;
	}
	
	if(streqnc(desc, "BindWebFormIP")){
    	dynncpy(copyconfig.bindWebFormIp,value,sizeof(copyconfig.bindWebFormIp));
    	return;
	}
    	
	if(streqnc(desc, "weblistenport")){
    	copyconfig.WebPort=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "linkpass")){
    	dynncpy(copyconfig.linkpass,value,sizeof(copyconfig.linkpass));
    	return;
	}
    	
	if(streqnc(desc, "WebSocketPass")){
    	dynncpy(copyconfig.WebSocketPass,value,sizeof(copyconfig.WebSocketPass));
    	return;
	}
    	
	if(streqnc(desc, "serverlink")){
    	copyconfig.serverlink=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "uline")){
    	addUline(thread,value);
    	return;
	}
    	
	if(streqnc(desc, "servername")){
    	dynncpy(copyconfig.servername,value,sizeof(copyconfig.servername));
    	return;
	}
    	
	if(streqnc(desc, "description")){
    	dynncpy(copyconfig.description,value,sizeof(copyconfig.description));
    	return;
	}
    	
	if(streqnc(desc, "vhost")){
    	dynncpy(copyconfig.vhost,value,sizeof(copyconfig.vhost));
    	return;
	}
    	
	if(streqnc(desc, "modes")){
    	dynncpy(copyconfig.modes,value,sizeof(copyconfig.modes));
    	return;
	}

	if(streqnc(desc, "Serviceshost")){
    	dynncpy(copyconfig.host,value,sizeof(copyconfig.host));
    	return;
	}
    	
	if(streqnc(desc, "nick")){
    	dynncpy(copyconfig.nick,value,sizeof(copyconfig.nick));
    	return;
	}
    	
	if(streqnc(desc, "ident")){
    	dynncpy(copyconfig.ident,value,sizeof(copyconfig.ident));
    	return;
	}

	if(streqnc(desc, "homepage")){
    	dynncpy(copyconfig.homepage,value,sizeof(copyconfig.homepage));
    	return;
	}
    	
	if(streqnc(desc, "webformlink")){
    	dynncpy(copyconfig.LinkToWebform,value,sizeof(copyconfig.LinkToWebform));
    	return;
	}
    	
	if(streqnc(desc, "protocol")){
    	dynncpy(copyconfig.protocol,value,sizeof(copyconfig.protocol));
    	return;
	}
	
	if(streqnc(desc, "pidfilename")){
    	straddns(copyconfig.pidfilename,value,sizeof(copyconfig.pidfilename));
    	return;
	}
    	
	if(streqnc(desc, "freelogs")){
    	copyconfig.freelogs=1;
    	return;
	}
    	
	if(streqnc(desc, "keeplogs")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"Keeplogs\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.KeepLogs=expire(value);
    	return;
	}
    	
	if(streqnc(desc, "logfile")){
    	dynncpy(copyconfig.logfile,value,sizeof(copyconfig.logfile));
    	return;
	}

	if(streqnc(desc, "sqllogfile")){
    	dynncpy(copyconfig.sqllogfile,value,sizeof(copyconfig.sqllogfile));
    	return;
	}
    	
	if(streqnc(desc, "debuglogging")){
    	copyconfig.debuglogging=1;
    	return;
	}

	if(streqnc(desc, "debug")){
    	copyconfig.debug=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "debuglog")){
    	dynncpy(copyconfig.debuglog,value,sizeof(copyconfig.debuglog));
    	return;
	}
    	
	if(streqnc(desc, "quotelog")){
    	dynncpy(copyconfig.logquotes,value,sizeof(copyconfig.logquotes));
    	return;
	}
    	
	if(streqnc(desc, "AutoRecognizeModules")){
		copyconfig.AutoRecognizeModules=1;
    	return;
	}
		
	if(streqnc(desc, "LoadModulesConf")){
		dynncpy(copyconfig.LoadModulesConf,value,sizeof(copyconfig.LoadModulesConf));
    	return;
	}
    	
	if(streqnc(desc, "logchan")){
    	dynncpy(copyconfig.logchan,value,sizeof(copyconfig.logchan));
    	return;
	}
    	
	if(streqnc(desc, "network")){
    	dynncpy(copyconfig.network,value,sizeof(copyconfig.network));
    	return;
	}
    	
	if(streqnc(desc, "strictpasswords")){
    	copyconfig.StrictPasswords=1;
    	return;
	}
    	
	if(streqnc(desc, "badpasslimit")){
    	copyconfig.BadPassLimit=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "SecureIdentify")){
    	copyconfig.SecureIdentify=stoi(value);
    	return;
	}
	
	if(streqnc(desc, "readtimeout")){
    	copyconfig.ReadTimeout=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "readtimeout_us")){
    	copyconfig.ReadTimeout_us=stoi(value);
    	return;
	}
    
	if(streqnc(desc, "usewebform")){
    	copyconfig.UseWebForm=1;
    	return;
	}
		
	if(streqnc(desc, "usewhowas")){
    	copyconfig.UseWhowas=1;
    	return;
	}

	if(streqnc(desc, "keepwhowaslog")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"Keepwhowaslog\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.KeepWhowasLog = expire(value);
    	return;
	}
    	
	if(streqnc(desc, "limitcheck")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"limitcheck\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.LimitCheck = expire(value);
    	return;
	}
    	
	if(streqnc(desc, "trustdumptime")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"trustdumptime\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.TrustDumpTime = expire(value);
    	return;
	}
    	
	if(streqnc(desc, "expirechecktime")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"expirechecktime\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.ExpireCheckTime = expire(value);
    	return;
	}

	if(streqnc(desc, "nickusedcheck")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"nickusedcheck\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.NickUsedCheck = expire(value);
    	return;
	}
    	
	if(streqnc(desc, "forceforbidreason")){
    	copyconfig.ForceForbidReason=1;
    	return;
	}
    	
	if(streqnc(desc, "useprivmsg")){
    	copyconfig.UsePrivmsg=1;
    	return;
	}

	if(streqnc(desc, "usestrictprivmsg")){
    	copyconfig.UseStrictPrivMsg=1;
    	return;
	}
    	
	if(streqnc(desc, "globaloncycle")){
    	copyconfig.GlobalOnCycle=1;
    	return;
	}
    	
	if(streqnc(desc, "globaloncyclemessage")){
    	dynncpy(copyconfig.GlobalOnCycleMessage,value,sizeof(copyconfig.GlobalOnCycleMessage));
    	return;
	}
    	
	if(streqnc(desc, "globaloncycleup")){
    	dynncpy(copyconfig.GlobalOnCycleUp,value,sizeof(copyconfig.GlobalOnCycleUp));
    	return;
	}
    	
	if(streqnc(desc, "anonymousglobal")){
    	copyconfig.AnonymousGlobal=1;
    	return;
	}
    	
	if(streqnc(desc, "usemail")){
    	copyconfig.UseMail=1;
    	return;
	}
    	
	if(streqnc(desc, "sendmailpath")){
    	dynncpy(copyconfig.sendmailpath,value,sizeof(copyconfig.sendmailpath));
    	return;
	}
    	
	if(streqnc(desc, "sendfrom")){
    	dynncpy(copyconfig.SendFrom,value,sizeof(copyconfig.SendFrom));
    	return;
	}
    	
	if(streqnc(desc, "restrictmail")){
    	copyconfig.RestrictMail=1;
    	return;
	}
    	
	if(streqnc(desc, "dontquoteaddresses")){
    	copyconfig.DontQuoteAddresses=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefKill")){
    	copyconfig.NSDefKill=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefKillQuick")){
    	copyconfig.NSDefKillQuick=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefKillImmed")){
    	copyconfig.NSDefKillImmed=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefSecure")){
    	copyconfig.NSDefSecure=1;
    	return;
	}

	if(streqnc(desc, "NSDefPrivate")){
    	copyconfig.NSDefPrivate=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefHideEmail")){
    	copyconfig.NSDefHideEmail=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefHideUsermask")){
    	copyconfig.NSDefHideUsermask=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefHideStatus")){
    	copyconfig.NSDefHideStatus=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefHideQuit")){
    	copyconfig.NSDefHideQuit=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefMsg")){
    	copyconfig.NSDefMsg=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefAutoop")){
    	copyconfig.NSDefAutoop=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefIncognito")){
    	copyconfig.NSDefIncognito=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefMemoSignon")){
    	copyconfig.NSDefMemoSignon=1;
    	return;
	}
    	
	if(streqnc(desc, "NSDefMemoReceive")){
    	copyconfig.NSDefMemoReceive=1;
    	return;
	}
			
	if(streqnc(desc, "NSDefLanguage")){
    	copyconfig.language=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "NSEmailReg")){
    	copyconfig.NSEmailReg=1;
    	return;
	}
    	
	if(streqnc(desc, "NSRegDelay")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"NSRegDelay\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.NSRegDelay=expire(value);
    	return;

	}
    	
	if(streqnc(desc, "NSResendDelay")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"NSResendDelay\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.NSResendDelay=expire(value);
    	return;
	}
    	
	if(streqnc(desc, "NSRExpire")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"NSRExpire\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.NSRExpire=expire(value);
    	return;
	}
    	
	if(streqnc(desc, "NSWaitRegister")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"NSWaitRegister\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.NSWaitRegister=expire(value);
    	return;
	}
	
	if(streqnc(desc, "NSQuickKill")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"NSQuickKill\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.NSQuickKill=expire(value);
    	return;
	}
	
	if(streqnc(desc, "NSImmedKill")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"NSImmedKill\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.NSImmedKill=expire(value);
    	return;
	}
    	
	if(streqnc(desc, "NSExpire")){
    	dynncpy(copyconfig.NSExpire,value,sizeof(copyconfig.NSExpire));
    	return;
	}
   
	if(streqnc(desc, "NSMaxGroups")){
    	copyconfig.NSMaxGroups=stoi(value);
    	return;
	}
	
	if(streqnc(desc, "NSMaxAliases")){
    	copyconfig.NSMaxAliases=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "NSAccessMax")){
    	copyconfig.NSAccessMax=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "NSReleaseTimeout")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"NSReleaseTimeout\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.NSReleaseTimeout=expire(value);
    	return;
	}
    	
	if(streqnc(desc, "NSNoGroupChange")){
    	copyconfig.NSNoGroupChange=1;
    	return;
	}
    	
	if(streqnc(desc, "NSListOpersOnly")){
    	copyconfig.NSListOpersOnly=1;
    	return;
	}
    	
	if(streqnc(desc, "NSListMax")){
    	copyconfig.NSListMax=stoi(value);
    	return;
	}
		
	if(streqnc(desc, "NSGuestNickPrefix")){
    	dynncpy(copyconfig.NSGuestNickPrefix,value,sizeof(copyconfig.NSGuestNickPrefix));
    	return;
	}
		
	if(streqnc(desc, "NSSecureAdmins")){
    	copyconfig.NSSecureAdmins=1;
    	return;
	}
			
	if(streqnc(desc, "NSRestrictGetPass")){
    	copyconfig.NSRestrictGetPass=1;
    	return;
	}
    	
	if(streqnc(desc, "NSUseTempLinks")){
    	copyconfig.NSUseTempLinks=1;
    	return;
	}	
    	
	if(streqnc(desc, "CSDefNone")){
    	copyconfig.CSDefNone=1;
    	return;
	}
    	
	if(streqnc(desc, "CSDefKeepTopic")){
    	copyconfig.CSDefKeepTopic=1;
    	return;
	}

	if(streqnc(desc, "CSDefOpNotice")){
    	copyconfig.CSDefOpNotice=1;
    	return;
	}
    	
	if(streqnc(desc, "CSDefPeace")){
    	copyconfig.CSDefPeace=1;
    	return;
	}

	if(streqnc(desc, "CSDefPrivate")){
    	copyconfig.CSDefPrivate=1;
    	return;
	}
    	
	if(streqnc(desc, "CSDefRestricted")){
    	copyconfig.CSDefRestricted=1;
    	return;
	}

	if(streqnc(desc, "CSDefSecure")){
    	copyconfig.CSDefSecure=1;
    	return;
	}

	if(streqnc(desc, "CSDefSecureOps")){
    	copyconfig.CSDefSecureOps=1;
    	return;
	}
    	
	if(streqnc(desc, "CSDefSecureFounder")){
    	copyconfig.CSDefSecureFounder=1;
    	return;
	}

	if(streqnc(desc, "CSDefSignKick")){
    	copyconfig.CSDefSignKick=1;
    	return;
	}
    	
	if(streqnc(desc, "CSDefSignKickLevel")){
    	copyconfig.CSDefSignKickLevel=1;
    	return;
	}

	if(streqnc(desc, "CSDefTopicLock")){
    	copyconfig.CSDefTopicLock=1;
    	return;
	}

	if(streqnc(desc, "CSDefXOP")){
    	copyconfig.CSDefXOP=1;
    	return;
	}
    	
	if(streqnc(desc, "CSDefInhabit")){
    	copyconfig.CSDefInhabit=1;
    	return;
	}

	if(streqnc(desc, "CSMaxReg")){
    	copyconfig.CSMaxReg=stoi(value);
    	return;
	}

	if(streqnc(desc, "CSWaitRegister")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"CSWaitRegister\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.CSWaitRegister=expire(value);
    	return;
	}
    	
	if(streqnc(desc, "CSExpire")){
    	dynncpy(copyconfig.CSExpire,value,sizeof(copyconfig.CSExpire));
    	return;
	}
    	
	if(streqnc(desc, "CSDefBantype")){
    	copyconfig.CSDefBantype=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "CSStayChannel")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"CSStayChannel\" proof your syntax, program exited\n");
			_orion_exit();
		}
		if(expire(value)<300)
    		copyconfig.CSStayChannel=300;
    	else
    		copyconfig.CSStayChannel=expire(value);
    	return;
	}

	if(streqnc(desc, "CSAccessMax")){
    	copyconfig.CSAccessMax=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "CSAutokickMax")){
    	copyconfig.CSAutokickMax=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "CSAutokickReason")){
    	dynncpy(copyconfig.CSAutokickReason,value,sizeof(copyconfig.CSAutokickReason));
    	return;
	}
    	
	if(streqnc(desc, "CSAutokickForceReason")){
    	copyconfig.CSAutokickForceReason=1;
    	return;
	}
    	
	if(streqnc(desc, "CSListOpersOnly")){
    	copyconfig.CSListOpersOnly=1;
    	return;
	}

	if(streqnc(desc, "CSListMax")){
    	copyconfig.CSListMax=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "CSOpersOnly")){
    	copyconfig.CSOpersOnly=1;
    	return;
	}
    	
	if(streqnc(desc, "MSMaxMemos")){
    	copyconfig.maxmemos=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "MSUsePriorities")){
    	copyconfig.MSUsePriorities=1;
    	return;
	}

	if(streqnc(desc, "BSDefDontKickOps")){
    	copyconfig.BSDefDontKickOps=1;
    	return;
	}

	if(streqnc(desc, "BSDefDontKickVoices")){
    	copyconfig.BSDefDontKickVoices=1;
    	return;
	}
    	
	if(streqnc(desc, "BSDefGreet")){
    	copyconfig.BSDefGreet=1;
    	return;
	}

	if(streqnc(desc, "BSDefFantasy")){
    	copyconfig.BSDefFantasy=1;
    	return;
	}
    	
	if(streqnc(desc, "BSDefSymbiosis")){
    	copyconfig.BSDefSymbiosis=1;
    	return;
	}
    	
	if(streqnc(desc, "BSMinUsers")){
    	copyconfig.BSMinUsers=stoi(value);
    	return;
	}

	if(streqnc(desc, "BSBadWordsMax")){
    	copyconfig.BSBadWordsMax=stoi(value);
    	return;
	}
    	
	if(streqnc(desc, "BSGentleBWReason")){
    	copyconfig.BSGentleBWReason=1;
    	return;
	}
    	
	if(streqnc(desc, "BSCaseSensitive")){
    	copyconfig.BSCaseSensitive=1;
    	return;
	}
    	
	if(streqnc(desc, "BSFantasyCharacter")){
    	dynncpy(copyconfig.BSFantasyCharacter,value,sizeof(copyconfig.BSFantasyCharacter));
    	return;
	}
    	
	if(streqnc(desc, "HSWaitRequest")){
		dynncpy(copyconfig.HSWaitRequest,value,sizeof(copyconfig.HSWaitRequest));
    	return;
	}
    	
	if(streqnc(desc, "socklinetime")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"socklinetime\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.socklinetime=expire(value);
    	return;
	}
    	
	if(streqnc(desc, "glineonclearchan")){
    	dynncpy(copyconfig.glineonclearchan,value,sizeof(copyconfig.glineonclearchan));
    	return;
	}
    	
	if(streqnc(desc, "GlineExpiry")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"GlineExpiry\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.GlineExpiry=expire(value);
    	return;
	}
    	
	if(streqnc(desc, "OSForceTrusts")){
    	copyconfig.OSForceTrusts=1;
    	return;
	}

	if(streqnc(desc, "OSGlineOnTrustViolation")){
    	copyconfig.OSGlineOnTrustViolation=1;
    	return;
	}
    	
	if(streqnc(desc, "OSForceTrustDelete")){
    	copyconfig.OSForceTrustDelete=1;
    	return;
	}
    	
	if(streqnc(desc, "OSUseRNlineIDline")){
    	copyconfig.OSUseRNlineIDline=1;
    	return;
	}
    
	if(streqnc(desc, "OSMsgFloodKill")){
		if(stol(value) == 0)
			copyconfig.OSMsgFloodKill=0;
		else{
			if(!check_expire(value)){
				dlog(thread,0,"CONFIG ERROR: Wrong time input for \"OSMsgFloodKill\" proof your syntax, program exited\n");
				_orion_exit();
			}
    		copyconfig.OSMsgFloodKill=expire(value);
		}
    	return;
	}
	
	if(streqnc(desc, "OSMsgFloodProt")){
    	copyconfig.OSMsgFloodProt=1;
    	return;
	}
    	
	if(streqnc(desc, "OSMsgFloodlines")){
    	copyconfig.OSMsgFloodlines=stol(value);
    	return;
	}
    	
	if(streqnc(desc, "OSMsgFloodtime")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"OSMsgFloodtime\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.OSMsgFloodtime=expire(value);
    	return;
	}
    
	if(streqnc(desc, "OSNickFloodProt")){
    	copyconfig.OSNickFloodProt=1;
    	return;
	}
    	
	if(streqnc(desc, "OSNickFloodlines")){
    	copyconfig.OSNickFloodlines=stol(value);
    	return;
	}
    	
	if(streqnc(desc, "OSNickFloodtime")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"OSNickFloodtime\" proof your syntax, program exited\n");
			_orion_exit();
		}
    	copyconfig.OSNickFloodtime=expire(value);
    	return;
	}

	if(streqnc(desc, "DisableRaw")){
    	copyconfig.DisableRaw=1;
    	return;
	}

	if(streqnc(desc, "WallBadOS")){
    	copyconfig.WallBadOS=1;
    	return;
	}

	if(streqnc(desc, "WallOSGlobal")){
    	copyconfig.WallOSGlobal=1;
    	return;
	}

	if(streqnc(desc, "WallOSMode")){
    	copyconfig.WallOSMode=1;
    	return;
	}

	if(streqnc(desc, "WallOSClearchan")){
    	copyconfig.WallOSClearchan=1;
    	return;
	}

	if(streqnc(desc, "WallOSKick")){
    	copyconfig.WallOSKick=1;
    	return;
	}

	if(streqnc(desc, "WallOSGline")){
    	copyconfig.WallOSGline=1;
    	return;
	}

	if(streqnc(desc, "WallOSNoOp")){
    	copyconfig.WallOSNoOp=1;
    	return;
	}

	if(streqnc(desc, "WallOSJupe")){
    	copyconfig.WallOSJupe=1;
    	return;
	}

	if(streqnc(desc, "WallOSRaw")){
    	copyconfig.WallOSRaw=1;
    	return;
	}

	if(streqnc(desc, "WallGetpass")){
    	copyconfig.WallGetpass=1;
    	return;
	}

	if(streqnc(desc, "WallSetpass")){
    	copyconfig.WallSetpass=1;
    	return;
	}

	if(streqnc(desc, "WallForbid")){
    	copyconfig.WallForbid=1;
    	return;
	}

	if(streqnc(desc, "WallDrop")){
    	copyconfig.WallDrop=1;
    	return;
	}
	
	if(streqnc(desc, "DefSessionLimit")){
		copyconfig.maxconnections=stol(value);
    	return;
	}
		
	if(streqnc(desc, "SessionLimitExceeded")){
		dynncpy(copyconfig.SessionLimitExceeded,value,sizeof(copyconfig.SessionLimitExceeded));
    	return;
	}
    	
	if(streqnc(desc, "MaxSessionKill")){
		copyconfig.MaxSessionKill=stoi(value);
    	return;
	}
		
	if(streqnc(desc, "SessionGlineExpiry")){
		if(!check_expire(value)){
			dlog(thread,0,"CONFIG ERROR: Wrong time input for \"SessionGlineExpiry\" proof your syntax, program exited\n");
			_orion_exit();
		}
		copyconfig.SessionGlineExpiry=expire(value);
    	return;
	}

	if(streqnc(desc, "OSOpersOnly")){
		copyconfig.OSOpersOnly=1;
    	return;
	}
		
	if(streqnc(desc, "DefConChanModes")){
		dynncpy(copyconfig.DefConChanModes,value,sizeof(copyconfig.DefConChanModes));
    	return;
	}

	if(streqnc(desc, "GlobalOnDefcon")){
    	copyconfig.GlobalOnDefcon=1;
    	return;
	}

	if(streqnc(desc, "DefconMessage")){
    	dynncpy(copyconfig.DefconMessage,value,sizeof(copyconfig.DefconMessage));
    	return;
	}

	if(streqnc(desc, "DefConOffMessage")){
    	dynncpy(copyconfig.DefConOffMessage,value,sizeof(copyconfig.DefConOffMessage));
    	return;
	}

	if(streqnc(desc, "DefConKillReason")){
    	dynncpy(copyconfig.DefConKillReason,value,sizeof(copyconfig.DefConKillReason));
    	return;
	}
    	
	if(streqnc(desc, "mysqlsrv")){
    	dynncpy(copyconfig.mysqlsrv,value,sizeof(copyconfig.mysqlsrv));
    	return;
	}
    	
	if(streqnc(desc, "mysqluser")){
    	dynncpy(copyconfig.mysqluser,value,sizeof(copyconfig.mysqluser));
    	return;
	}
    	
	if(streqnc(desc, "mysqlpass")){
    	dynncpy(copyconfig.mysqlpass,value,sizeof(copyconfig.mysqlpass));
    	return;
	}
    	
	if(streqnc(desc, "mysqldb")){
    	dynncpy(copyconfig.mysqlDB,value,sizeof(copyconfig.mysqlDB));
    	return;
	}
    	
	if(streqnc(desc, "mysqlport")){
    	copyconfig.mysqlport=stol(value);
    	return;
	}
	
	if(streqnc(desc, "mysqlsocket")){
		dynncpy(copyconfig.mysqlsocket,value,sizeof(copyconfig.mysqlsocket));
    	return;
	}

	if(streqnc(desc, "mysqlsuperuser")){
		copyconfig.MySQLSuperUser=stoi(value);
    	return;
	}
	
	if(streqnc(desc, "MysqlMove")){
		dynncpy(copyconfig.MysqlMove,value,sizeof(copyconfig.MysqlMove));
    	return;
	}

	if(streqnc(desc, "NewEncryption")){
		dynncpy(copyconfig.NewEncrypt,value,sizeof(copyconfig.NewEncrypt));
    	return;
	}

	if(streqnc(desc, "Encryption")){
		dynncpy(copyconfig.encrypt,value,sizeof(copyconfig.encrypt));
    	return;
	}
		
/**************************************************************************************/
/*************************** SERVICES CLIENTS *****************************************/
/**************************************************************************************/

/*
 *      nick NickServ
 *      ident NickServ
 *      realname NickName Server
 *      host NickServ
 */
	if(streqnc(desc, "NSNick")){
		dynncpy(ns_conf_l.nick,value,sizeof(ns_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "NSIdent")){
		dynncpy(ns_conf_l.ident,value,sizeof(ns_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "NSRealname")){
		dynncpy(ns_conf_l.realname,value,sizeof(ns_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "NSHost")){
		dynncpy(ns_conf_l.host,value,sizeof(ns_conf_l.host));
    	return;
	}
	if(streqnc(desc, "NSMinPasswordlen")){
		ns_conf_l.min_passwordlen=(size_t)stol(value);
    	return;
	}

/**************************************************************************************/

/*
 *      nick BotServ
 *      ident BotServ
 *      realname Bot Server
 *      host BotServ
 */
	if(streqnc(desc, "BSNick")){
		dynncpy(bs_conf_l.nick,value,sizeof(bs_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "BSIdent")){
		dynncpy(bs_conf_l.ident,value,sizeof(bs_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "BSRealname")){
		dynncpy(bs_conf_l.realname,value,sizeof(bs_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "BSHost")){
		dynncpy(bs_conf_l.host,value,sizeof(bs_conf_l.host));
    	return;
	}

/**************************************************************************************/

/*
 *      nick ChanServ
 *      ident ChanServ
 *      realname Channel Server
 *      host ChanServ
 */
	if(streqnc(desc, "CSNick")){
		dynncpy(cs_conf_l.nick,value,sizeof(cs_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "CSIdent")){
		dynncpy(cs_conf_l.ident,value,sizeof(cs_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "CSRealname")){
		dynncpy(cs_conf_l.realname,value,sizeof(cs_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "CSHost")){
		dynncpy(cs_conf_l.host,value,sizeof(cs_conf_l.host));
    	return;
	}

/**************************************************************************************/

/*
 *      nick MemoServ
 *      ident MemoServ
 *      realname Memo Server
 *      host MemoServ
 */
	if(streqnc(desc, "MSNick")){
		dynncpy(ms_conf_l.nick,value,sizeof(ms_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "MSIdent")){
		dynncpy(ms_conf_l.ident,value,sizeof(ms_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "MSRealname")){
		dynncpy(ms_conf_l.realname,value,sizeof(ms_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "MSHost")){
		dynncpy(ms_conf_l.host,value,sizeof(ms_conf_l.host));
    	return;
	}

/**************************************************************************************/

/*
 *      nick HostServ
 *      ident HostServ
 *      realname vHost Server
 *      host HostServ.*
 */
	if(streqnc(desc, "HSNick")){
		dynncpy(hs_conf_l.nick,value,sizeof(hs_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "HSIdent")){
		dynncpy(hs_conf_l.ident,value,sizeof(hs_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "HSRealname")){
		dynncpy(hs_conf_l.realname,value,sizeof(hs_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "HSHost")){
		dynncpy(hs_conf_l.host,value,sizeof(hs_conf_l.host));
    	return;
	}

/**************************************************************************************/

/*
 *      nick O
 *      ident operserv
 *      realname This is operservice 2.36
 *      host operserv.*
 */
	if(streqnc(desc, "OSNick")){
		dynncpy(O_conf_l.nick,value,sizeof(O_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "OSIdent")){
		dynncpy(O_conf_l.ident,value,sizeof(O_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "OSRealname")){
		dynncpy(O_conf_l.realname,value,sizeof(O_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "OSHost")){
		dynncpy(O_conf_l.host,value,sizeof(O_conf_l.host));
    	return;
	}

/**************************************************************************************/

/*
 *      nick Q
 *      ident TheQBot
 *      realname The Q Bot
 *      host CServe.*
 */
	if(streqnc(desc, "QNick")){
		dynncpy(Q_conf_l.nick,value,sizeof(Q_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "QIdent")){
		dynncpy(Q_conf_l.ident,value,sizeof(Q_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "QRealname")){
		dynncpy(Q_conf_l.realname,value,sizeof(Q_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "QHost")){
		dynncpy(Q_conf_l.host,value,sizeof(Q_conf_l.host));
    	return;
	}

/**************************************************************************************/

/*
 *      nick sockscontrol
 *      ident sockscontrol
 *      realname Sockscontrol_v0.1
 *      host proxy.*
 *      sockmode 0
 *      sockcons 3
 *      sockseconds 5
 */
	if(streqnc(desc, "SCNick")){
		dynncpy(sc_conf_l.nick,value,sizeof(sc_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "SCIdent")){
		dynncpy(sc_conf_l.ident,value,sizeof(sc_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "SCRealname")){
		dynncpy(sc_conf_l.realname,value,sizeof(sc_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "SCHost")){
		dynncpy(sc_conf_l.host,value,sizeof(sc_conf_l.host));
    	return;
	}
	if(streqnc(desc, "Sockmode")){
		sc_conf_l.sockmode=stol(value);
    	return;
	}
	if(streqnc(desc, "Sockcons")){
		sc_conf_l.sockcons=stol(value);
    	return;
	}
	if(streqnc(desc, "Sockseconds")){
		sc_conf_l.sockseconds=stol(value);
    	return;
	}
	if(streqnc(desc, "ResetDefconSeconds")){
		sc_conf_l.reset_defcon_seconds=stol(value);
    	return;
	}
	if(streqnc(desc, "AutoDefcon1")){
		sc_conf_l.auto_defcon_1=stol(value);
    	return;
	}
	if(streqnc(desc, "AutoDefcon2")){
		sc_conf_l.auto_defcon_2=stol(value);
    	return;
	}
	if(streqnc(desc, "AutoDefcon3")){
		sc_conf_l.auto_defcon_3=stol(value);
    	return;
	}
	if(streqnc(desc, "AutoDefcon4")){
		sc_conf_l.auto_defcon_4=stol(value);
    	return;
	}
		
/**************************************************************************************/

/*
 *      nick Global
 *      ident Global
 *      realname Global @ DeepIRC.net
 *      host services.deepirc.net
 */
	if(streqnc(desc, "GNick")){
		dynncpy(g_conf_l.nick,value,sizeof(g_conf_l.nick));
    	return;
	}
	if(streqnc(desc, "GIdent")){
		dynncpy(g_conf_l.ident,value,sizeof(g_conf_l.ident));
    	return;
	}
	if(streqnc(desc, "GRealname")){
		dynncpy(g_conf_l.realname,value,sizeof(g_conf_l.realname));
    	return;
	}
	if(streqnc(desc, "GHost")){
		dynncpy(g_conf_l.host,value,sizeof(g_conf_l.host));
    	return;
	}

/**************************************************************************************/

	return;
}

/*  ************************************************************************************************  */

/**
 * @brief moduleLoadconfig:  Loads an external Module Config
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param Module The module for which we like to add the config strings (to delete it on module unload)
 * @param configfile The configfile name
 * @return void
 **/
void moduleLoadconfig(Threads *thread, ModCo *module, char *configfile){ //
	/* Declare variables */
	FILE *in;
	char path[MAX_FILEPATH_LEN];
	char line[1000];
	char desc[1000];
	char value[1000];
	char *array[4];

	/* Initialise variables to (NULL) */
	bzero(&path,sizeof(path));
	
	
	snprintf(path,MAX_FILEPATH_LEN,"%s/",SERVICES_CONF_DIR);
	straddns(path,configfile,sizeof(path));

	dlog(thread,3,"moduleLoadconfig(); path: %s\n",path);

	in = fopen(path,"r");

	if(in==NULL){
		dlog(thread,0,"ERROR: Configfile not found !! - Check the path to %s\n",configfile);
		dlog(thread,0,"Tried path was \"%s\"\n",path);
		_orion_exit();
	}

	bzero(&line,sizeof(line));
	while(fgets(line,sizeof(line)-1,in)){
		if(line[0]!='#' && line[1]!='\n'){
			bzero(&desc,sizeof(desc));
			bzero(&value,sizeof(value));
			bzero(array,sizeof(array));
			strclean(line);

			if(strcin('\"',line)){
				sscanf(line,"%s \"%s[^\"]",desc,value);
			}else{
				sscanf(line,"%s %s",desc,value);
				strclean(value);
				strclean(desc);
			}

			addModuleConfig(thread,desc,value,module->filename);
		}
		else if(streq(line,"#EOF")) break;
	}
	
	dynncpy(module->configfile,configfile,sizeof(module->configfile));
	fclose(in);
	return;
}
