/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_banlist.c
 *   @brief This is the Module file for ChanServ BANLIST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ BANLIST command.
 *
 *   The ChanServ BANLIST Command allows the user to dump all Bans<br/>
 *    from target #channel
 *
 *   Syntax is: /msg ChanServ BANLIST #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "cs"
#define MODUL "banlist"
#define MODUL_FILE "cs_banlist"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_banlist(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_banlist_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_banlist_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by <br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"banlist","banlist");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"banlist",cs_banlist,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help banlist",cs_banlist_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_banlist_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ BANLIST Command.
 * 
 * The ChanServ BANLIST Command allows the user to dump all Bans<br/>
 *   from target #channel
 *  
 * Syntax is: /msg ChanServ BANLIST #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_banlist(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL banlist;
	//ChannelDB c;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&banlist,sizeof(banlist));
	//bzero(&c,sizeof(c));

	/*
	 * Banlist Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"banlist")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.ban){
				banlist.res    = doSelectMySQL(thread,"SELECT `ban` FROM `orion_chan_bans` WHERE `channel`='%s'",mysql_quote(thread,param[4]));
        		banlist.fields = mysql_num_rows(banlist.res);
				
                for(i=0;i<banlist.fields;i++){
                    banlist.row = mysql_fetch_row(banlist.res);
                    
					send_lang(thread,language,howtosend,snick,nick,"CS_BANLIST_LIST_BANS",i+1,banlist.row[0]);
                }
                send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
                
				mysql_free_result_myr(banlist.res); //free MySQL result
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ BANLIST Command (/msg ChanServ HELP BANLIST)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP BANLIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP BANLIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_banlist_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Banlist Help
	 */
	if(thread->sql.access.rights<ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help banlist")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_BANLIST",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - BANLIST part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with <br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the BANLIST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_banlist_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Banlist (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_BANLIST");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
