/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_set.c
 *   @brief This is the Module file for ChanServ SET Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ SET command.
 *
 *   The ChanServ SET Command allows the user to set various SET-Options<br/>
 *   for target Channel. (Controls FOUNDER/SUCCESSOR etc.)
 *
 *   Syntax is: /msg ChanServ SET #channel OPTION ...
 *
 *<table border="0">
 *    <tr><td> FOUNDER			</td><td> Syntax is: /msg ChanServ SET #channel FOUNDER			</td><td> < nick >							</td></tr>
 *    <tr><td> SUCSESSOR		</td><td> Syntax is: /msg ChanServ SET #channel SUCSESSOR		</td><td> < nick >							</td></tr>
 *    <tr><td> MLOCK			</td><td> Syntax is: /msg ChanServ SET #channel MLOCK			</td><td> < mlock (i.e: +ntlk-fc 10 rofl) >	</td></tr>
 *    <tr><td> DESCRIPTION		</td><td> Syntax is: /msg ChanServ SET #channel DESCRIPTION		</td><td> < new-description >				</td></tr>
 *    <tr><td> URL				</td><td> Syntax is: /msg ChanServ SET #channel URL				</td><td> < new-Url >						</td></tr>
 *    <tr><td> EMAIL			</td><td> Syntax is: /msg ChanServ SET #channel EMAIL			</td><td> < new-Email >						</td></tr>
 *    <tr><td> ENTRYMSG			</td><td> Syntax is: /msg ChanServ SET #channel ENTRYMSG		</td><td> < new-EntryMsg >					</td></tr>
 *    <tr><td> BANTYPE			</td><td> Syntax is: /msg ChanServ SET #channel BANTYPE			</td><td> < new-Bantype(0-3) >				</td></tr>
 *    <tr><td> KEEPTOPIC		</td><td> Syntax is: /msg ChanServ SET #channel KEEPTOPIC		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> OPNOTICE			</td><td> Syntax is: /msg ChanServ SET #channel OPNOTICE		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> PEACE			</td><td> Syntax is: /msg ChanServ SET #channel PEACE			</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> PRIVATE			</td><td> Syntax is: /msg ChanServ SET #channel PRIVATE			</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> RESTRICTED		</td><td> Syntax is: /msg ChanServ SET #channel RESTRICTED		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> SECURE			</td><td> Syntax is: /msg ChanServ SET #channel SECURE			</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> SECUREOPS		</td><td> Syntax is: /msg ChanServ SET #channel SECUREOPS		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> SECUREFOUNDER	</td><td> Syntax is: /msg ChanServ SET #channel SECUREFOUNDER	</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> SIGNKICK			</td><td> Syntax is: /msg ChanServ SET #channel SIGNKICK		</td><td> { ON | LEVEL | OFF }				</td></tr>
 *    <tr><td> TOPICLOCK		</td><td> Syntax is: /msg ChanServ SET #channel TOPICLOCK		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> XOP				</td><td> Syntax is: /msg ChanServ SET #channel XOP				</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> NOEXPIRE			</td><td> Syntax is: /msg ChanServ SET #channel NOEXPIRE		</td><td> { ON | OFF }						</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"
#include "webform.h"	//orion_WRCtranslate_flood_mode_check();

#define SERVICE "cs"
#define MODUL "set"
#define MODUL_FILE "cs_set"
#define MODUL_VERSION "1.2"

/*(02:45) -ChanServ- FOUNDER Set the founder of a channel
(02:45) -ChanServ- SUCCESSOR Set the successor for a channel
(02:45) -ChanServ- DESC Set the channel description
(02:45) -ChanServ- URL Associate a URL with the channel
(02:45) -ChanServ- EMAIL Associate an E-mail address with the channel
(02:45) -ChanServ- ENTRYMSG Set a message to be sent to users when they
(02:45) -ChanServ- enter the channel
(02:45) -ChanServ- BANTYPE Set how Services make bans on the channel
(02:45) -ChanServ- MLOCK Lock channel modes on or off
(02:45) -ChanServ- KEEPTOPIC Retain topic when channel is not in use
(02:45) -ChanServ- OPNOTICE Send a notice when OP/DEOP commands are used
(02:45) -ChanServ- PEACE Regulate the use of critical commands
(02:45) -ChanServ- PRIVATE Hide channel from LIST command
(02:45) -ChanServ- RESTRICTED Restrict access to the channel
(02:45) -ChanServ- SECURE Activate ChanServ security features
(02:45) -ChanServ- SECUREOPS Stricter control of chanop status
(02:45) -ChanServ- SECUREFOUNDER Stricter control of channel founder status
(02:45) -ChanServ- SIGNKICK Sign kicks that are done with KICK command
(02:45) -ChanServ- TOPICLOCK Topic can only be changed with TOPIC
(02:45) -ChanServ- XOP Toggle the user privilege system*/

MOD_BOOL cs_set_founder(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_successor(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_description(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_url(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_entrymsg(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_bantype(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_mlock(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_keeptopic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_opnotice(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_peace(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_private(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_restricted(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_secure(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_secureops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_securefounder(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_signkick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_topiclock(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_xop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_inhabit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_noexpire(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//--------------------------------------------------------------------------------------

MOD_BOOL cs_set_founder_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_successor_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_description_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_url_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_entrymsg_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_bantype_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_mlock_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_keeptopic_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_opnotice_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_peace_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_private_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_restricted_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_secure_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_secureops_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_securefounder_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_signkick_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_topiclock_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_xop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_inhabit_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_noexpire_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL cs_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"set * option[1]","set option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"set",is_services_admin,is_services_user); /* just to add a displaylevel for /msg cs help
															  this should always be the lowest used Module level! */
															  
	addCommand(thread,&Module,"set founder",cs_set_founder,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set successor",cs_set_successor,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set desc",cs_set_description,is_services_admin,is_services_user,6,0);
	addCommand(thread,&Module,"set url",cs_set_url,is_services_admin,is_services_user,5,6);
	addCommand(thread,&Module,"set email",cs_set_email,is_services_admin,is_services_user,5,6);
	addCommand(thread,&Module,"set entrymsg",cs_set_entrymsg,is_services_admin,is_services_user,5,0);
	addCommand(thread,&Module,"set bantype",cs_set_bantype,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set mlock",cs_set_mlock,is_services_admin,is_services_user,4,0);
	addCommand(thread,&Module,"set keeptopic",cs_set_keeptopic,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set opnotice",cs_set_opnotice,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set peace",cs_set_peace,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set private",cs_set_private,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set restricted",cs_set_restricted,is_services_admin,is_services_user,6,6);
	//addCommand(thread,&Module,"set secure",cs_set_secure,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set secureops",cs_set_secureops,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set securefounder",cs_set_securefounder,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set signkick",cs_set_signkick,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set topiclock",cs_set_topiclock,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set xop",cs_set_xop,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set inhabit",cs_set_inhabit,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set noexpire",cs_set_noexpire,is_services_admin,is_services_user,6,6);
	
	addCommand(thread,&Module,"help set founder",cs_set_founder_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set successor",cs_set_successor_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set desc",cs_set_description_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set url",cs_set_url_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set email",cs_set_email_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set entrymsg",cs_set_entrymsg_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set bantype",cs_set_bantype_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set mlock",cs_set_mlock_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set keeptopic",cs_set_keeptopic_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set opnotice",cs_set_opnotice_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set peace",cs_set_peace_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set private",cs_set_private_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set restricted",cs_set_restricted_help,is_services_admin,is_services_user,5,5);
	//addCommand(thread,&Module,"help set secure",cs_set_secure_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set secureops",cs_set_secureops_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set securefounder",cs_set_securefounder_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set signkick",cs_set_signkick_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set topiclock",cs_set_topiclock_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set xop",cs_set_xop_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set inhabit",cs_set_inhabit_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set noexpire",cs_set_noexpire_help,is_services_admin,is_services_admin,5,5);
	
	addCommand(thread,&Module,"help set",cs_set_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_set_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Founder function for the ChanServ SET Command.
 * 
 * The ChanServ SET Founder Command allows the user to change the FounderNick<br/>
 *   to a new FounderNick
 *  
 * Syntax is: /msg ChanServ SET #channel FOUNDER < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_founder(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//Channel c;
	//Aliases aliases5;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	//bzero(&aliases5,sizeof(aliases5));

	/*
	 * Set Founder Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set founder")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || (stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER && ~thread->c.chan.settings&CS_SECUREFOUNDER_BIT) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set founder")){
				//aliases5 = sql_aliases(thread,param[6]);
				
				if(thread->c.aliases.authnick[0]!='\0'){
    				if(!thread->c.aliases.forbid){
						/* Update Memory, for caching */
						dynncpy(thread->c.chan.founder,thread->c.aliases.authnick,sizeof(thread->c.chan.founder));
						
						/* Update SQL as Live Update */
    					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`founder`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name));
    					
    					if(stoi(thread->c.chan.access.targetlevel)!=0)
    						domysql(thread,"UPDATE `orion_cs_access` SET `level`='%d' WHERE `authnick`='%s' AND `channel`='%s' AND `serv`='ChanServ'",ACCESS_FOUNDER,mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name));
    					else
    						domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('ChanServ','%s','%s','%d')",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name),ACCESS_FOUNDER);
    						
    					send_lang(thread,language,howtosend,snick,nick,"CS_SET_FOUNDER_SUCCESSFUL_SET",thread->c.aliases.authnick,thread->c.chan.name);
    					
    					setWebSuccess(thread); /* Indentify webform that changes were applied */
    				}else send_lang(thread,language,howtosend,snick,nick,"NICK_SUSPENDED_OR_FORBIDDEN",thread->c.aliases.nick);
                }else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Successor function for the ChanServ SET Command.
 * 
 * The ChanServ SET Successor Command allows the user to change the SuccessorNick<br/>
 *   to a new SuccessorNick
 *  
 * Syntax is: /msg ChanServ SET #channel SUCSESSOR < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_successor(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//Channel c;
	//Aliases aliases5;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	//bzero(&aliases5,sizeof(aliases5));

	/*
	 * Set Successor Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set successor")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || (stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER && ~thread->c.chan.settings&CS_SECUREFOUNDER_BIT) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set successor")){
				//aliases5 = sql_aliases(thread,param[6]);
				
				if(thread->c.aliases.authnick[0]!='\0'){
    				if(!thread->c.aliases.forbid){
						/* Update Memory, for caching */
						dynncpy(thread->c.chan.successor,thread->c.aliases.authnick,sizeof(thread->c.chan.successor));
						dynncpy(thread->c.chan.access.targetlevel,_dynadd("%d",ACCESS_SUCCESSOR),sizeof(thread->c.chan.access.targetlevel));
						
						/* Update SQL as Live Update */
    					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`successor`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name));
    					
    					if(stoi(thread->c.chan.access.targetlevel)!=0)
    						domysql(thread,"UPDATE `orion_cs_access` SET `level`='%d' WHERE `authnick`='%s' AND `channel`='%s' AND `serv`='ChanServ'",ACCESS_SUCCESSOR,mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name));
    					else 
    						domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('ChanServ','%s','%s','%d')",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name),ACCESS_SUCCESSOR);
    					
    					send_lang(thread,language,howtosend,snick,nick,"CS_SET_SUCCESSOR_SUCCESSFUL_SET",thread->c.aliases.authnick,thread->c.chan.name);
    					
    					setWebSuccess(thread); /* Indentify webform that changes were applied */
    				}else send_lang(thread,language,howtosend,snick,nick,"NICK_SUSPENDED_OR_FORBIDDEN");
                }else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Mlock function for the ChanServ SET Command.
 * 
 * The ChanServ SET Mlock Command allows the user to change the Mlock<br/>
 *   for target Channel
 *  
 * Syntax is: /msg ChanServ SET #channel MLOCK < mlock (i.e: +ntlk-fc 10 rofl) >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_mlock(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	char modes[35];
	char positive[35];
	char negative[35];

	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	bzero(&modes,sizeof(modes));
	bzero(&positive,sizeof(positive));
	bzero(&negative,sizeof(negative));

	/*
	 * Set Mlock Command
	 */
	 //:shock PRIVMSG ChanServ :set #channel mlock +O
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set mlock")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set mlock")){
	        	if(dynlen(param[6])==1 && param[6][0]=='+'){
        			domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`mlock_flood`='',`mlock_key`='',`mlock_limit`='',`mlock_positive`='',`mlock_negative`='' WHERE `channel`='%s'",time(NULL),mysql_quote(thread,thread->c.chan.name));
					bzero(&thread->c,sizeof(thread->c));
				}
				else if(dynlen(param[6])>1 && (param[6][0]=='+' || param[6][0]=='-')){         	
            		dynncpy(modes,param[6],sizeof(modes));
		           	
            		int nr=7;
					size_t i=0;
		           	bool type=0;
            		for(i=0;i<dynlen(modes);i++){
		           		char current=modes[i];
		           		//printf("i = %d\n",i);
		           		//printf("mode = %c\n",current);
		           		
            			if(current=='+') type=1;
		           		else if(current=='-') type=0;
            			else if(findchar(modes,current)>0){
            				if(type==1){
		           				if(strcin(current,IRC.ParametricChannelModes)){
									if(!strcin(current,positive)){
										straddnc(positive,current,sizeof(positive));
									}
									
            						if(current==IRC.FloodCMode){
										bool flood_mode_valid = orion_WRCtranslate_flood_mode_check(thread,param[nr]);
										
										/* Update Memory, for caching */
										if(flood_mode_valid == true){
											dynncpy(thread->c.chan.mlock_flood,param[nr],sizeof(thread->c.chan.mlock_flood));
										}else
											strdelc(positive,current);
										
                                        /* Update SQL as Live Update */
                                        if(flood_mode_valid == true)
		           							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`mlock_flood`='%s' WHERE `channel`='%s'",time(NULL),mysql_quote(thread,param[nr]),mysql_quote(thread,thread->c.chan.name));
									}else if(current==IRC.KeyCMode){
										/* Update Memory, for caching */
										dynncpy(thread->c.chan.mlock_key,param[nr],sizeof(thread->c.chan.mlock_key));

                                        /* Update SQL as Live Update */
            							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`mlock_key`='%s' WHERE `channel`='%s'",time(NULL),mysql_quote(thread,param[nr]),mysql_quote(thread,thread->c.chan.name));
									}else if(current==IRC.LimitCMode){
										/* Update Memory, for caching */
										thread->c.chan.mlock_limit = stol(param[nr]);

                                        /* Update SQL as Live Update */
		           						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`mlock_limit`='%ld' WHERE `channel`='%s'",time(NULL),stol(param[nr]),mysql_quote(thread,thread->c.chan.name));
									}else if(current==IRC.LinkCMode){
										/* Update Memory, for caching */
										dynncpy(thread->c.chan.mlock_redirect,param[nr],sizeof(thread->c.chan.mlock_redirect));

                                        /* Update SQL as Live Update */
		           						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`mlock_redirect`='%s' WHERE `channel`='%s'",time(NULL),mysql_quote(thread,param[nr]),mysql_quote(thread,thread->c.chan.name));
									}
									nr++;
            					}else{
            						if(!strcin(current,positive))
										straddnc(positive,current,sizeof(positive));
								}
		           			}else if(type==0){
            					if(strcin(current,IRC.ParametricChannelModes)){
            						if(thread->c.chan.fields && !strcin(current,negative))
										straddnc(negative,current,sizeof(negative));
		           				}
								else{
            						if(thread->c.chan.fields && !strcin(current,negative) && !strcin(current,positive))
										straddnc(negative,current,sizeof(negative));
            					}
		           			}
		           		}else{
							send_lang(thread,language,howtosend,snick,nick,"CS_SET_MLOCK_INVALID_MODES");
							return MOD_CONT;
						}
            		}
            		sort(negative);
            		sort(positive);
            		
            		/* Update Memory, for caching */
					dynncpy(thread->c.chan.mlock_negative,negative,sizeof(thread->c.chan.mlock_negative));
					dynncpy(thread->c.chan.mlock_positive,positive,sizeof(thread->c.chan.mlock_positive));
            		
                    /* Update SQL as Live Update */
            		domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`mlock_negative`='%s',`mlock_positive`='%s' WHERE `channel`='%s'",time(NULL),mysql_quote(thread,negative),mysql_quote(thread,positive),mysql_quote(thread,thread->c.chan.name));
            		
            		if(param[7]) do_mode(thread,CBot,thread->c.chan.name,thread->c.chan.datecreate,_dynadd("%s %s",modes,_dstrjoin(' ',param,7)));
		           	else do_mode(thread,CBot,thread->c.chan.name,thread->c.chan.datecreate,modes);
              	}else if(!param[6]){
              		send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER");
              		return MOD_CONT;
				}else{
              		send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[6]);
              		return MOD_CONT;
				}
	    		send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
	    		
	    		setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Description function for the ChanServ SET Command.
 * 
 * The ChanServ SET Description Command allows the user to change<br/>
 *   the Channel's Description
 *  
 * Syntax is: /msg ChanServ SET #channel DESCRIPTION < new-description >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_description(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Description Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set desc")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set desc")){
				/* Update Memory, for caching */
				dynncpy(thread->c.chan.description,_dstrjoin(' ',param,6),sizeof(thread->c.chan.description));
				
				/* Update SQL as Live Update */
				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`description`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.description),mysql_quote(thread,thread->c.chan.name));
				send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Url function for the ChanServ SET Command.
 * 
 * The ChanServ SET Url Command allows the user to change<br/>
 *   the Channel's Url
 *  
 * Syntax is: /msg ChanServ SET #channel URL < new-Url >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_url(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Url Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set url")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set url")){
				if(!param[6]){
					/* Update Memory, for caching */
					bzero(&thread->c.chan.url,sizeof(thread->c.chan.url));

					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`url`='' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(!is_valid_chars(param[6],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-_.:/%")){
					/* Update Memory, for caching */
					dynncpy(thread->c.chan.url,param[6],sizeof(thread->c.chan.url));

					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`url`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,param[6]),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Email function for the ChanServ SET Command.
 * 
 * The ChanServ SET Email Command allows the user to change<br/>
 *   the Channel's Email-Address.
 *  
 * Syntax is: /msg ChanServ SET #channel EMAIL < new-Email >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Set Email Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set email")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set email")){
				if(param[6]){
					//dynncpy(email,param[6],sizeof(email));

					/*if(!strcsea(param[6],"@&.")){
						send_lang(thread,language,howtosend,snick,nick,"CS_SET_EMAIL_INVALID_EMAIL");
						return MOD_CONT;
					}
		
					strsplit(email,domain,'@');
					dynncpy(emailname,domain[0],sizeof(emailname));
					strsplit(domain[1],parts,'.');
					domain[0] = emailname;

					dlog(thread,3,"domain[0] = %s\n",domain[0]);
					dlog(thread,3,"domain[1] = %s\n",domain[1]);*/
		
					if(!MailValidate(param[6])){
						send_lang(thread,language,howtosend,snick,nick,"CS_SET_EMAIL_INVALID_EMAIL");
						return MOD_CONT;
					}
		
					else{// if(dynlen(parts[0])>=ns->min_emaildomainlen && dynlen(parts[1])>=ns->min_emaildomainlen){ //email ok and accepted
						/* Update Memory, for caching */
						dynncpy(thread->c.chan.email,param[6],sizeof(thread->c.chan.email));

						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`email`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,param[6]),mysql_quote(thread,thread->c.chan.name));
						send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
						
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}
					//else send_lang(thread,language,howtosend,snick,nick,"CS_SET_EMAIL_INVALID_EMAIL");
				}else{
					/* Update Memory, for caching */
					bzero(&thread->c.chan.email,sizeof(thread->c.chan.email));

					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`email`='' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET EntryMsg function for the ChanServ SET Command.
 * 
 * The ChanServ SET EntryMsg Command allows the user to change<br/>
 *   the Channel's EntryMsg.
 *  
 * Syntax is: /msg ChanServ SET #channel ENTRYMSG < new-EntryMsg >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_entrymsg(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Entrymsg Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set entrymsg")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set entrymsg")){
				if(param[6]){
					/* Update Memory, for caching */
					dynncpy(thread->c.chan.welcome,_dstrjoin(' ',param,6),sizeof(thread->c.chan.welcome));

					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`welcome`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.welcome),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else{
					/* Update Memory, for caching */
					bzero(&thread->c.chan.welcome,sizeof(thread->c.chan.welcome));

					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`welcome`='' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Bantype function for the ChanServ SET Command.
 * 
 * The ChanServ SET Bantype Command allows the user to change<br/>
 *   the Channel's Bantype.
 *  
 * Syntax is: /msg ChanServ SET #channel BANTYPE < new-Bantype(0-3) >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_bantype(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Bantype Command
	 */
	if(digitcol(param[6]) && dynlen(param[6])==1 && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set bantype")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set bantype")){

				/* Update Memory, for caching */
				if(stoi(param[6])==0){
					thread->c.chan.settings &= ~CS_BANTYPE_1_BIT;
					thread->c.chan.settings &= ~CS_BANTYPE_2_BIT;
				}else if(stoi(param[6])==1){
					thread->c.chan.settings |=  CS_BANTYPE_1_BIT;
					thread->c.chan.settings &= ~CS_BANTYPE_2_BIT;
				}else if(stoi(param[6])==2){
					thread->c.chan.settings &= ~CS_BANTYPE_1_BIT;
					thread->c.chan.settings |=  CS_BANTYPE_2_BIT;
				}else if(stoi(param[6])==3){
					thread->c.chan.settings |=  CS_BANTYPE_1_BIT;
					thread->c.chan.settings |=  CS_BANTYPE_2_BIT;
				}
				else{
					send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[6]);
					return MOD_CONT;
				}
				
				/* Update SQL as Live Update */
				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
				send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		if(!digitcol(param[6]) || dynlen(param[6])>1)													send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[6]);
		else if(thread->sql.access.rights<ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set bantype"))	send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		else																							send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET KeepTopic function for the ChanServ SET Command.
 * 
 * The ChanServ SET KeepTopic Command allows the user to permanently<br/>
 *   set the Channel's Topithread->c.chan.
 *  
 * Syntax is: /msg ChanServ SET #channel KEEPTOPIC { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_keeptopic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Keeptopic Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set keeptopic")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set keeptopic")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_KEEPTOPIC_BIT){
						/* Update Memory, for caching */
						thread->c.chan.settings |= CS_KEEPTOPIC_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_KEEPTOPIC_BIT){
						/* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_KEEPTOPIC_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Opnotice function for the ChanServ SET Command.
 * 
 * The ChanServ SET Opnotice Command allows the user to set<br/>
 *   Opnotice on/off (Notices ChannelOps on Op/Deop of other Users)
 *  
 * Syntax is: /msg ChanServ SET #channel OPNOTICE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_opnotice(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Opnotice Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set opnotice")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set opnotice")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_OPNOTICE_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings |= CS_OPNOTICE_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_OPNOTICE_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_OPNOTICE_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Peace function for the ChanServ SET Command.
 * 
 * The ChanServ SET Peace Command allows the user to set<br/>
 *   Peace on/off<br/>
 *   When Peace is set, an user won't be able to kick,<br/>
 *	 ban or remove a channel status of an user that has<br/>
 *	 a level superior or equal to his via ChanServ commands
 *  
 * Syntax is: /msg ChanServ SET #channel PEACE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_peace(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Peace Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set peace")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set peace")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_PEACE_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings |= CS_PEACE_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_PEACE_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_PEACE_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Private function for the ChanServ SET Command.
 * 
 * The ChanServ SET Private Command allows the user to set<br/>
 *   Private on/off<br/>
 *	 Enables or disables the Private option for a channel.<br/>
 *	 When Private is set, a ChanServ LIST will not<br/>
 *	 include the channel in any lists.
 *  
 * Syntax is: /msg ChanServ SET #channel PRIVATE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_private(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Private Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set private")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set private")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_PRIVATE_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings |= CS_PRIVATE_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_PRIVATE_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_PRIVATE_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Restricted function for the ChanServ SET Command.
 * 
 * The ChanServ SET Restricted Command allows the user to set<br/>
 *   Restricted on/off<br/>
 *	 Enables or disables the RESTRICTED option for a<br/>
 *	 channel.  When RESTRICTED is set, users who would<br/>
 *	 normally be disallowed from having channel operator<br/>
 *	 privileges (users with negative access levels and, if<br/>
 *	 SECUREOPS is set, users not on the access list) will<br/>
 *	 instead be kicked and banned from the channel.
 *  
 * Syntax is: /msg ChanServ SET #channel RESTRICTED { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_restricted(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Restricted Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set restricted")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set restricted")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_RESTRICTED_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings |= CS_RESTRICTED_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_RESTRICTED_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_RESTRICTED_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Secure function for the ChanServ SET Command.
 * 
 * The ChanServ SET Secure Command allows the user to set<br/>
 *   Secure on/off<br/>
 *	 Enables or disables ChanServ's security features for a<br/>
 *	 channel.  When SECURE is set, only users who have<br/>
 *	 registered their nicknames with NickServ and IDENTIFY'd<br/>
 *	 with their password will be given access to the channel<br/>
 *	 as controlled by the access list.
 *  
 * Syntax is: /msg ChanServ SET #channel SECURE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_secure(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Secure Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set secure")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set secure")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_SECURE_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings |= CS_SECURE_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_SECURE_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_SECURE_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET SecureOps function for the ChanServ SET Command.
 * 
 * The ChanServ SET SecureOps Command allows the user to set<br/>
 *   SecureOps on/off<br/>
 *	 Enables or disables the SECUREOPS option for a channel.<br/>
 *	 When SECUREOPS is set, users who are not on the userlist<br/>
 *	 will not be allowed chanop status.
 *  
 * Syntax is: /msg ChanServ SET #channel SECUREOPS { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_secureops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set SecureOps Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set secureops")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set secureops")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_SECUREOPS_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings |= CS_SECUREOPS_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_SECUREOPS_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_SECUREOPS_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET SecureFounder function for the ChanServ SET Command.
 * 
 * The ChanServ SET SecureFounder Command allows the user to set<br/>
 *   SecureFounder on/off<br/>
 *	 Enables or disables the SecureFounder option for a channel.<br/>
 *	 When SecureFounder is set, only the real founder will be<br/>
 *	 able to drop the channel, change its password, its founder and its<br/>
 *	 successor.
 *  
 * Syntax is: /msg ChanServ SET #channel SECUREFOUNDER { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_securefounder(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Securefounder Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set securefounder")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || (stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER && ~thread->c.chan.settings&CS_SECUREFOUNDER_BIT) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set securefounder")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_SECUREFOUNDER_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings |= CS_SECUREFOUNDER_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_SECUREFOUNDER_BIT){
                        /* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_SECUREFOUNDER_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Signkick function for the ChanServ SET Command.
 * 
 * The ChanServ SET Signkick Command allows the user to set<br/>
 *   Signkick on/off
 *
 *  When SIGNKICK is set, kicks issued with<br/>
 * 	a KICK command will have the nick that used the<br/>
 * 	command in their reason.
 * 	
 * 	If you use LEVEL, those who have a level that is superior<br/>
 * 	or equal to the SIGNKICK level on the channel won't have their<br/>
 * 	kicks signed. See ChanServ HELP LEVELS for more information.
 *  
 * Syntax is: /msg ChanServ SET #channel SIGNKICK { ON | LEVEL | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_signkick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Signkick Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set signkick")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set signkick")){
				if(streqnc(param[6],"on")){
					/* Update Memory, for caching */
					if(~thread->c.chan.settings&CS_SIGNKICK_1_BIT) thread->c.chan.settings |= CS_SIGNKICK_1_BIT;
					if(thread->c.chan.settings&CS_SIGNKICK_2_BIT) thread->c.chan.settings &= ~CS_SIGNKICK_2_BIT;
					
					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
 					/* Update Memory, for caching */
					if(thread->c.chan.settings&CS_SIGNKICK_1_BIT) thread->c.chan.settings &= ~CS_SIGNKICK_1_BIT;
					if(thread->c.chan.settings&CS_SIGNKICK_2_BIT) thread->c.chan.settings &= ~CS_SIGNKICK_2_BIT;
					
					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"level")){
					/* Update Memory, for caching */
					if(~thread->c.chan.settings&CS_SIGNKICK_1_BIT) thread->c.chan.settings |= CS_SIGNKICK_1_BIT;
					if(~thread->c.chan.settings&CS_SIGNKICK_2_BIT) thread->c.chan.settings |= CS_SIGNKICK_2_BIT;
					
					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET TopicLock function for the ChanServ SET Command.
 * 
 * The ChanServ SET TopicLock Command allows the user to set<br/>
 *   TopicLock on/off
 *
 * 	Enables or disables the TopicLock option for a channel.<br/>
 * 	When TopicLock is set, ChanServ will not allow the<br/>
 * 	channel topic to be changed except via the TOPIC<br/>
 * 	command.
 *  
 * Syntax is: /msg ChanServ SET #channel TOPICLOCK { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_topiclock(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set TopicLock Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set topiclock")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set topiclock")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_TOPICLOCK_BIT){
						/* Update Memory, for caching */
						thread->c.chan.settings |= CS_TOPICLOCK_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_TOPICLOCK_BIT){
						/* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_TOPICLOCK_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Xop function for the ChanServ SET Command.
 * 
 * The ChanServ SET Xop Command allows the user to set<br/>
 *   Xop on/off
 *
 * 	Enables or disables the xOP lists system for a channel.<br/>
 * 	When XOP is set, you have to use the AOP/SOP/VOP/HOP<br/>
 * 	commands in order to give channel privileges to<br/>
 * 	users, else you have to use the ACCESS command.
 * 	
 * 	Technical Note:<br/>
 *	when you switch from access list to xOP<br/>
 * 	lists system, your level definitions and user levels will be<br/>
 * 	changed, so you won't find the same values if you<br/>
 * 	switch back to access system!
 *
 * 	You should also check that your users are in the good xOP<br/>
 * 	list after the switch from access to xOP lists, because the<br/>
 * 	guess is not always perfect... in fact, it is not recommended<br/>
 * 	to use the xOP lists if you changed level definitions with<br/>
 * 	the LEVELS command.
 *
 * 	Switching from xOP lists system to access list system<br/>
 * 	causes no problem though.
 *  
 * Syntax is: /msg ChanServ SET #channel XOP { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_xop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Xop Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set xop")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set xop")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_XOP_BIT){
						/* Update Memory, for caching */
						thread->c.chan.settings |= CS_XOP_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_XOP_BIT){
						/* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_XOP_BIT;
						
						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Inhabit function for the ChanServ SET Command.
 *
 * The ChanServ SET Inhabit Command allows the user to set<br/>
 *   Inhabit on/off
 *
 *  Sets whether ChanServ permanantly Inhabits the channel or not.<br/>
 *  Setting this to ON makes ChanServ inhabit the channel permanently.
 *
 * Syntax is: /msg ChanServ SET #channel INHABIT { ON | OFF }
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_inhabit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Inhabit Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set inhabit")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);

		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set inhabit")){
				if(streqnc(param[6],"on")){
					if(~thread->c.chan.settings&CS_INHABIT_BIT){
						/* Update Memory, for caching */
						thread->c.chan.settings |= CS_INHABIT_BIT;

						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
						
						/* Make ChanServ join the channel */
						do_services_join(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate == 0 ? time(NULL) : thread->c.chan.datecreate,2);
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
					if(thread->c.chan.settings&CS_INHABIT_BIT){
						/* Update Memory, for caching */
						thread->c.chan.settings &= ~CS_INHABIT_BIT;

						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`settings`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
						
						/* Make ChanServ part the channel */
						do_part(thread,cs->nick,thread->c.chan.name,"inhabit off");
					}
					send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET NoExpire function for the ChanServ SET Command.
 * 
 * The ChanServ SET NoExpire Command allows the user to set<br/>
 *   NoExpire on/off
 *
 *  Sets whether the given channel will expire.  Setting this<br/>
 *  to ON prevents the channel from expiring.
 *
 *  Limited to Services admins.
 *  
 * Syntax is: /msg ChanServ SET #channel NOEXPIRE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_noexpire(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//Channel c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set NoExpire Command
	 */
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set noexpire")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){

			/* Update Memory, for caching */
			if(streqnc(param[6],"on")) thread->c.chan.expire=0;
			else if(streqnc(param[6],"off")) thread->c.chan.expire=1;

			/* Update SQL as Live Update */
	    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`expire`='%d' WHERE `channel`='%s' AND `serv`!=''",time(NULL),thread->c.chan.expire,mysql_quote(thread,thread->c.chan.name));
    		send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ SET Command (/msg ChanServ HELP SET)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set")) {
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET",snick,snick);
		if(thread->sql.access.rights>ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set"))
			send_lang(thread,language,howtosend,snick,nick,"CS_SERVADMIN_HELP_SET");
			
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Founder Help function for the ChanServ SET Command (/msg ChanServ HELP SET FOUNDER)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET FOUNDER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET FOUNDER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_founder_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Founder Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set founder")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_FOUNDER");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Successor Help function for the ChanServ SET Command (/msg ChanServ HELP SET SUCCESSOR)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET SUCCESSOR,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET SUCCESSOR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_successor_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Successor Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set successor")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_SUCCESSOR",30);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Descripton Help function for the ChanServ SET Command (/msg ChanServ HELP SET DESCRIPTION)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET DESCRIPTION,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET DESCRIPTION
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_description_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Descripton Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set desc")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_DESC");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Url Help function for the ChanServ SET Command (/msg ChanServ HELP SET URL)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET URL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET URL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_url_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Url Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set url")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_URL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Email Help function for the ChanServ SET Command (/msg ChanServ HELP SET EMAIL)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET EMAIL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET EMAIL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Email Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set email")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_EMAIL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET EntryMsg Help function for the ChanServ SET Command (/msg ChanServ HELP SET ENTRYMSG)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET ENTRYMSG,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET ENTRYMSG
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_entrymsg_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set EntryMsg Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set entrymsg")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_ENTRYMSG");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Bantype Help function for the ChanServ SET Command (/msg ChanServ HELP SET BANTYPE)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET BANTYPE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET BANTYPE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_bantype_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Bantype Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set bantype")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_BANTYPE");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Mlock Help function for the ChanServ SET Command (/msg ChanServ HELP SET MLOCK)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET MLOCK,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET MLOCK
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_mlock_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Mlock Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set mlock")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_MLOCK",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Keeptopic Help function for the ChanServ SET Command (/msg ChanServ HELP SET KEEPTOPIC)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET KEEPTOPIC,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET KEEPTOPIC
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_keeptopic_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Keeptopic Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set keeptopic")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_KEEPTOPIC",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET OpNotice Help function for the ChanServ SET Command (/msg ChanServ HELP SET OPNOTICE)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET OPNOTICE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET OPNOTICE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_opnotice_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set OpNotice Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set opnotice")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_OPNOTICE",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Peace Help function for the ChanServ SET Command (/msg ChanServ HELP SET PEACE)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET PEACE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET PEACE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_peace_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Peace Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set peace")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_PEACE",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Private Help function for the ChanServ SET Command (/msg ChanServ HELP SET PRIVATE)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET PRIVATE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET PRIVATE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_private_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Private Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set private")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_PRIVATE",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Restricted Help function for the ChanServ SET Command (/msg ChanServ HELP SET RESTRICTED)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET RESTRICTED,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET RESTRICTED
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_restricted_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Restricted Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set restricted")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_RESTRICTED");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET SecureOps Help function for the ChanServ SET Command (/msg ChanServ HELP SET SECUREOPS)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET SECUREOPS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET SECUREOPS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_secureops_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set SecureOps Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set secureops")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_SECUREOPS");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Secure Help function for the ChanServ SET Command (/msg ChanServ HELP SET SECURE)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET SECURE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET SECURE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_secure_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Secure Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set secure")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_SECURE",snick,ns->nick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET SecureFounder Help function for the ChanServ SET Command (/msg ChanServ HELP SET SECUREFOUNDER)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET SECUREFOUNDER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET SECUREFOUNDER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_securefounder_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set SecureFounder Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set securefounder")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_SECUREFOUNDER");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET SignKick Help function for the ChanServ SET Command (/msg ChanServ HELP SET SIGNKICK)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET SIGNKICK,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET SIGNKICK
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_signkick_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set SignKick Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set signkick")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_SIGNKICK",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET TopicLock Help function for the ChanServ SET Command (/msg ChanServ HELP SET TOPICLOCK)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET TOPICLOCK,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET TOPICLOCK
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_topiclock_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set TopicLock Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set topiclock")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_TOPICLOCK",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Xop Help function for the ChanServ SET Command (/msg ChanServ HELP SET XOP)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET XOP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET XOP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_xop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Xop Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set xop")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_XOP");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Inhabit Help function for the ChanServ SET Command (/msg ChanServ HELP SET INHABIT)
 *
 * When the user tries to get help for this command, using /msg ChanServ HELP SET INHABIT,<br/>
 * this function is executed.
 *
 * Syntax: /msg ChanServ HELP SET INHABIT
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_inhabit_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Inhabit Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set inhabit")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_INHABIT",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET NoExpire Help function for the ChanServ SET Command (/msg ChanServ HELP SET NOEXPIRE)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET NOEXPIRE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET NOEXPIRE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_noexpire_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set NoExpire Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set noexpire")){
		send_lang(thread,language,howtosend,snick,nick,"CS_SERVADMIN_HELP_SET_NOEXPIRE");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - SET part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SET command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Set (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_SET");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
