/*  ************************************************************************************************
 *
 *  THIS PART WAS TAKEN FROM "Anope" (C) Anope & Daniel Engel (Modified for Orion)
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file enc_none.c
 *   @brief This is the Module file for Encrypting Password's (NoEncryption)
 *
 *   It contains all functions which are neccessary to encrypt/decrypt
 *   Passwords
 */
/*  ************************************************************************************************  */

#include "encrypt.h"
#include "modules.h"
#include "services.h"
#include "webform.h"

#define SERVICE "enc"
#define MODUL "none"
#define MODUL_FILE "enc_none"
#define MODUL_VERSION "1.2"

int plain_encrypt(const char *src, size_t len, char *dest, size_t size);
int plain_encrypt_in_place(char *buf, size_t size);
int plain_encrypt_check_len(size_t passlen, size_t bufsize);
int plain_decrypt(const char *src, char *dest, size_t size);
int plain_check_password(const char *plaintext, const char *password);
char* plain_encryption(void);

MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by 
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,NULL,NULL);

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addAccess (thread,&Module,NOONE);
	addMODType(thread,&Module,ENCRYPT);

    encmodule_encrypt(plain_encrypt);
    encmodule_encrypt_in_place(plain_encrypt_in_place);
    encmodule_encrypt_check_len(plain_encrypt_check_len);
    encmodule_decrypt(plain_decrypt);
    encmodule_check_password(plain_check_password);
    encmodule_check_encryption(plain_encryption);
    
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

int plain_encrypt(const char *src, size_t len, char *dest ,size_t size) {
    if(size>=len) {
	memset(dest,0,size);
        strncpy(dest,src,len);
        dest[len] = '\0';
	return 0;
    }
    return -1;
}

int plain_encrypt_in_place(char *buf, size_t size) {
    return 0;
}

int plain_encrypt_check_len(size_t passlen, size_t bufsize) {
    if(bufsize>=passlen) {
        return 0;
    }
    return bufsize;
}

int plain_decrypt(const char *src, char *dest, size_t size) {
    memset(dest,0,size);
    strncpy(dest,src,size);
    return 1;
}

int plain_check_password(const char *plaintext, const char *password) {
    if(strcmp(plaintext,password)==0) {
        return 1;
    }
    return 0;
}

char *plain_encryption(void){
	return "plain";
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
