/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_waiting.c
 *   @brief This is the Module file for HostServ WAITING Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the HostServ WAITING command.
 *
 *   The HostServ WAITING Command allows the user to perform a LIST +req.<br/>
 *   (Command was just added for convinience)
 *
 *   Syntax is: /msg HostServ WAITING
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "hs"
#define MODUL "waiting"
#define MODUL_FILE "hs_waiting"
#define MODUL_VERSION "1.2"

MOD_BOOL hs_waiting(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_waiting_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_waiting_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.VHost){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"waiting","waiting");

	addAuthor (thread,&Module,"Ally, cLiEbiG, (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"waiting",hs_waiting,is_services_oper,INFINIT,3,4);
	addCommand(thread,&Module,"help waiting",hs_waiting_help,is_services_oper,INFINIT,4,4);
	addCommand(thread,&Module,"help",hs_waiting_help_listing,is_services_oper,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ WAITING Command.
 * 
 * The HostServ WAITING Command allows the user to perform a LIST +req.<br/>
 *   (Command was just added for convinience)
 *
 * Syntax is: /msg HostServ WAITING
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_waiting(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    deepSQL vhosts;
    int i = 0;
    
	/* Initialise Variables to (NULL) */
	bzero(&vhosts,sizeof(vhosts));
    
	/*
	 * Waiting Command
	 */
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"waiting")){
    
       	send_lang(thread,language,howtosend,snick,nick,"HS_WAITING_START");   	
    	if(!param[4]){
    		vhosts.res    = doSelectMySQL(thread,"SELECT `nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active` FROM `orion_hs_core` WHERE `authorized`='' LIMIT 0,50");
    		vhosts.fields = mysql_num_rows(vhosts.res);
    		
    		if(vhosts.fields){
    			for(i=0;i<vhosts.fields;i++){
					vhosts.row = mysql_fetch_row(vhosts.res);
					
    				msend(thread,howtosend,snick,nick,"#%ld: Nickname: %s Display: %s vHost: %s (Requested on %s)",i+1,vhosts.row[0],vhosts.row[1],vhosts.row[3],get_localtime((time_t)stol(vhosts.row[5])));
    			}
    			send_lang(thread,language,howtosend,snick,nick,"HS_WAITING_END",1,i-1,vhosts.fields);
    		}else send_lang(thread,language,howtosend,snick,nick,"HS_WAITING_NOENTRIES_END");
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
    	}else{
    		if(stoi(param[4]) == 0)
    			param[4] = "1";
    
    		vhosts.res    = doSelectMySQL(thread,"SELECT `nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active` FROM `orion_hs_core` WHERE `authorized`='' LIMIT %ld,%ld",stol(param[4])-1,stol(param[4])+50);
    		vhosts.fields = mysql_num_rows(vhosts.res);
    		
    		if(vhosts.fields){
    			for(i=0;i<vhosts.fields;i++){
					vhosts.row = mysql_fetch_row(vhosts.res);
					
    				msend(thread,howtosend,snick,nick,"#%ld: Nickname: %s Display: %s vHost: %s (Requested on %s)",i+stol(param[4]),vhosts.row[0],vhosts.row[1],vhosts.row[3],get_localtime((time_t)stol(vhosts.row[5])));
    			}
    			send_lang(thread,language,howtosend,snick,nick,"HS_WAITING_END",stol(param[4]),i+stol(param[4])-1,vhosts.fields);
    		}else send_lang(thread,language,howtosend,snick,nick,"HS_WAITING_NOENTRIES_END");
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
		}
		
		/* mysql_free_result_myr(); - vhosts */
		mysql_free_result_myr(vhosts.res);
    }else{
	/*
	 * Error Messages
	 */
 		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ WAITING Command (/msg HostServ HELP WAITING)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP WAITING,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP WAITING
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_waiting_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Waiting Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help waiting")){
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_WAITING",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - WAITING part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the WAITING command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_waiting_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Waiting (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_WAITING");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
