/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_list.c
 *   @brief This is the Module file for MemoServ LIST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ LIST command.
 *
 *   The MemoServ LIST Command allows the user to dump the memos he/she currently has.<br/>
 *   With "NEW", dumps only new (unread) memos.  Unread memos are marked with a "*"<br/>
 *   to the left of the memo number.
 *
 *   Syntax is: /msg MemoServ LIST [#channel] [NEW]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "ms"
#define MODUL "list"
#define MODUL_FILE "ms_list"
#define MODUL_VERSION "1.2"

MOD_BOOL ms_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_list_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"list","list");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"list",ms_list,is_services_admin,is_services_user,3,5);
	addCommand(thread,&Module,"help list",ms_list_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_list_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//:shock PRIVMSG MemoServ :list
/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ LIST Command.
 * 
 * The MemoServ LIST Command allows the user to dump the memos he/she currently has.<br/>
 *   With "NEW", dumps only new (unread) memos.  Unread memos are marked with a "*"<br/>
 *   to the left of the memo number.
 *
 * Syntax is: /msg MemoServ LIST [#channel] [NEW]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Gobal Variables */
	deepSQL memos;
	ChannelDB c;
	int i=0;
	int memocount = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&memos,sizeof(memos));
	bzero(&c,sizeof(c));
	
	/*
	 * List Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list")){
        if(!param[4]){
            memos.res    = doSelectMySQL(thread,"SELECT `time`,`sender`,`memo`,`mode` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` ASC",mysql_quote(thread,thread->sql.aliases.authnick));
            memos.fields = mysql_num_rows(memos.res);
            
            if(memos.fields){
                send_lang(thread,language,howtosend,snick,nick,"MS_LIST_HEADER",thread->sql.aliases.authnick,snick);
                
                for(i=0;i<memos.fields;i++){
					memos.row = mysql_fetch_row(memos.res);
					
                    if(stoi(memos.row[3])==0) msend(thread,howtosend,snick,nick,"* %d %s %s",i+1,memos.row[1],get_localtime((time_t)stol(memos.row[0])));
                    else msend(thread,howtosend,snick,nick,"%d %s %s",i+1,memos.row[1],get_localtime((time_t)stol(memos.row[0])));
                }
                
                send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
                
                setWebSuccess(thread); /* Indentify webform that changes were applied */
            }else send_lang(thread,language,howtosend,snick,nick,"MS_NO_MEMOS_FOR_TARGET",thread->sql.aliases.authnick);
            
    		/* mysql_free_result_myr(); - memos */
    		mysql_free_result_myr(memos.res);
        }else{
            i=4;
            int NEW=0, channel=0;
    		while(param[i]){
                if(streqnc(param[i],"new")) NEW=i;
                else if(param[i][0]=='#') channel=i;
                i++;
    		}
    		if(NEW && !channel){
				memos.res    = doSelectMySQL(thread,"SELECT `time`,`sender`,`memo`,`mode` FROM `orion_ms_core` WHERE `receiver`='%s' AND `mode`='0' ORDER BY `time` ASC",mysql_quote(thread,thread->sql.aliases.authnick));
            	memos.fields = mysql_num_rows(memos.res);

        		if(memos.fields>0){
                    send_lang(thread,language,howtosend,snick,nick,"MS_LIST_HEADER",thread->sql.aliases.authnick,snick);
                    while(memocount < memos.fields){
                        memos.row = mysql_fetch_row(memos.res);

                        msend(thread,howtosend,snick,nick,"* %d %s %s",memocount+1,memos.row[1],get_localtime((time_t)stol(memos.row[0])));
                        memocount++;
                    }

                    send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
                    send_lang(thread,language,howtosend,snick,nick,"MS_LIST_NOTE_DO_NOT_DELETE_WITH_THESE_NUMBERS");
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
                }else send_lang(thread,language,howtosend,snick,nick,"MS_NO_MEMOS_FOR_TARGET",thread->sql.aliases.authnick);
                
        		/* mysql_free_result_myr(); - memos */
        		mysql_free_result_myr(memos.res);
    		}else if(NEW && channel){
                c = sql_chan_db(thread,param[channel]);
                
                if(c.serv[0]!='\0' && (stoi(acclevel(thread,"ChanServ",c.name,thread->sql.aliases.authnick))>=c.cs_memo || strcor(acclevel(thread,"Q",c.name,thread->sql.aliases.authnick),"o") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"list"))){
                    memos.res    = doSelectMySQL(thread,"SELECT `time`,`sender` FROM `orion_ms_core` WHERE `receiver`='%s'  AND `mode`='0' ORDER BY `time` ASC",mysql_quote(thread,c.name));
                    memos.fields = mysql_num_rows(memos.res);
                    
                    if(memos.fields){
                        send_lang(thread,language,howtosend,snick,nick,"MS_LIST_HEADER_CHANNEL",c.name,snick,c.name);
                        
                        for(i=0;i<memos.fields;i++){
                            memos.row = mysql_fetch_row(memos.res);
                            
                            msend(thread,howtosend,snick,nick,"* %d %s %s",i+1,memos.row[1],get_localtime((time_t)stol(memos.row[0])));
                        }
                        
                        send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
                        send_lang(thread,language,howtosend,snick,nick,"MS_LIST_NOTE_DO_NOT_DELETE_WITH_THESE_NUMBERS");
                        
                        setWebSuccess(thread); /* Indentify webform that changes were applied */
                    }else send_lang(thread,language,howtosend,snick,nick,"MS_NO_MEMOS_FOR_TARGET",c.name);
                    
            		/* mysql_free_result_myr(); - memos */
            		mysql_free_result_myr(memos.res);
                }else{
					if(streqnc(c.serv,"chanserv"))
						send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",stoi(acclevel(thread,"ChanServ",c.name,thread->sql.aliases.authnick)),c.name);
					else if(streqnc(c.serv,"q"))
						send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");	
				}
    		}
            else if(!NEW && channel){
                c = sql_chan_db(thread,param[channel]);
                 if(c.serv[0]!='\0' && (stoi(acclevel(thread,"ChanServ",c.name,thread->sql.aliases.authnick))>=c.cs_memo || strcor(acclevel(thread,"Q",c.name,thread->sql.aliases.authnick),"o") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"list"))){
                    memos.res    = doSelectMySQL(thread,"SELECT `time`,`sender`,`memo`,`mode` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` ASC",mysql_quote(thread,c.name));
                    memos.fields = mysql_num_rows(memos.res);
                    
                    if(memos.fields){
                        send_lang(thread,language,howtosend,snick,nick,"MS_LIST_HEADER_CHANNEL",c.name,snick,c.name);
                        
                        for(i=0;i<memos.fields;i++){
                            memos.row = mysql_fetch_row(memos.res);
                            
                            if(stoi(memos.row[3])==0) msend(thread,howtosend,snick,nick,"* %d %s %s",i+1,memos.row[1],get_localtime((time_t)stol(memos.row[0])));
                            else msend(thread,howtosend,snick,nick,"%d %s %s",i+1,memos.row[1],get_localtime((time_t)stol(memos.row[0])));
                        }
                        
                        send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
                        //send_lang(thread,language,howtosend,snick,nick,"MS_LIST_NOTE_DO_NOT_DELETE_WITH_THESE_NUMBERS");
                        
                        setWebSuccess(thread); /* Indentify webform that changes were applied */
                    }else send_lang(thread,language,howtosend,snick,nick,"MS_NO_MEMOS_FOR_TARGET",c.name);
                    
            		/* mysql_free_result_myr(); - memos */
            		mysql_free_result_myr(memos.res);
                }else{
					if(streqnc(c.serv,"chanserv"))
						send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",stoi(acclevel(thread,"ChanServ",c.name,thread->sql.aliases.authnick)),c.name);
					else if(streqnc(c.serv,"q"))
						send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");	
				}
            }
        }
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ LIST Command (/msg MemoServ HELP LIST)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list")){
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_LIST");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - LIST part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the LIST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_list_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * List (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_LIST");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
