/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_release.c
 *   @brief This is the Module file for NickServ RELEASE Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ RELEASE command.
 *
 *   The NickServ RELEASE Command allows the user to release target nickname.<br/>
 *   If the nick is your Nickname, but not in your group you have to provide < password ><br/>
 *   otherwise (the nick is in your group) you don't have to provie < password >.<br/>
 *   This Command Kills target user (if the nick seems to be yours).<br/>
 *   May be used to kill Fakeusers after using RECOVER.
 *
 *   Syntax is: /msg NickServ RELEASE < nick > [< password >]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "release"
#define MODUL_FILE "ns_release"
#define MODUL_VERSION "1.2"

MOD_BOOL ns_release(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_release_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_release_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"release","release");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,WEBACCESS,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"release",ns_release,is_user,is_user,4,5);
	addCommand(thread,&Module,"help release",ns_release_help,is_user,is_user,4,5);
	addCommand(thread,&Module,"help",ns_release_help_listing,is_user,is_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ RELEASE Command.
 * 
 * The NickServ RELEASE Command allows the user to release target nickname.<br/>
 *   If the nick is your Nickname, but not in your group you have to provide < password ><br/>
 *   otherwise (the nick is in your group) you don't have to provie < password >.<br/>
 *   This Command Kills target user (if the nick seems to be yours).<br/>
 *   May be used to kill Fakeusers after using RECOVER.
 *
 * Syntax is: /msg NickServ RELEASE < nick > [< password >]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_release(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access access5;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	
	/*
	 * Release Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"release")){ //cmd release
		if(!streqnc(nick,param[4])){
        	aliases5 = sql_aliases(thread,param[4]);
        	access5  = sql_access(thread,aliases5.authnick);
			
			if(streqnc(aliases5.authnick,thread->sql.aliases.authnick) && !param[5] && thread->sql.aliases.active){
				do_kill(thread,snick,param[4],"NickServ was sended a release command - which caused a kill");
				//do_sqline(thread,ns->nick,param[4],time(NULL)+60,"NickServ was sended a recover command - which caused a sqline");
				send_lang(thread,language,howtosend,snick,nick,"NS_COMMAND_DONE");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else if(param[5] && orion_check_password(param[5],access5.password)){
				do_kill(thread,snick,param[4],"NickServ was sended a release command - which caused a kill");
				send_lang(thread,language,howtosend,snick,nick,"NS_COMMAND_DONE");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"NICK_NOT_IN_YOUR_GROUP_OR_WRONG_PW");
		}else send_lang(thread,language,howtosend,snick,nick,"NS_RELEASE_YOU_CANT_RELEASE_YOURSELF");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ RELEASE Command (/msg NickServ HELP RELEASE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP RELEASE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP RELEASE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_release_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Release Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help release")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_RELEASE",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - RELEASE part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the RELEASE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_release_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Release (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_RELEASE");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
