/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_noop.c
 *   @brief This is the Module file for O NOOP Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O NOOP command.
 *
 *   The O NOOP Command allows an admin to remove all IRCOps from<br/>
 *   the Network.<br/>
 *   1. Noop Temp - removes the IRCOps temporarily (UMODE -o) - no further action<br/>
 *   2. Noop Perm - removes the IRCOps permanently (UMODE -o) - silently, if they<br/>
 *      try to re /oper they will be killed immediately. Ensures that noone on the network<br/>
 *      has IRCOp Status or is able to get it. (Useful when somebody hacked an ircop account)<br/>
 *      NOTE: In this case all O Commands are locked, even Services Admins can't execute<br/>
 *       ANY O command EXCEPT !NOOP PERM OFF! (Security Purpose) - Change all passwords IMMEDIATELY!
 *
 *   Syntax is: /msg O NOOP { TEMP | PERM } [ ON | OFF ]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "O"
#define MODUL "noop"
#define MODUL_FILE "o_noop"
#define MODUL_VERSION "1.2"

MOD_BOOL O_noop_temp(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_noop_perm(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_noop_temp_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_noop_perm_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_noop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_noop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"noop option[1]","noop option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"noop",NULL,960,INFINIT,0,0); /*just to add a displaylevel for /msg O help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"noop temp",O_noop_temp,960,INFINIT,4,4);
	addCommand(thread,&Module,"noop perm",O_noop_perm,1000,INFINIT,5,5);
	
	addCommand(thread,&Module,"help noop",O_noop_help,960,INFINIT,4,4);
	addCommand(thread,&Module,"help noop temp",O_noop_temp_help,960,INFINIT,5,5);
	addCommand(thread,&Module,"help noop perm",O_noop_perm_help,1000,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",O_noop_help_listing,960,INFINIT,3,3); //100 - each Admin is able to see it.
	
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O NOOP Temp function for the O NOOP Command.
 * 
 * The O NOOP Temp Command allows an admin to temporarily Noop all Ops
 *  
 * Syntax is: /msg O NOOP TEMP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_noop_temp(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL nmodes;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&nmodes,sizeof(nmodes));

	/*
	 * Noop Temp Command
	 */
	if(strcor("o",thread->sql.Cdata.modes) && thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"noop temp")){
		nmodes.res	  = doSelectMySQL(thread,"SELECT `nick`,`modes` FROM `orion_core_condata` WHERE `faketype`='0' AND `modes` RLIKE 'o'");
		nmodes.fields = mysql_num_rows(nmodes.res);
		
		/*if(m->WallOSNoOp)
			do_global(thread,O->nick,"NoOp was set from %s, removing all IRCOps.",nick);
			
			//Shouldn't it be silent !? - Changed by shock (25.05.2008)
		*/

		if(nmodes.fields){
			for(i=0;i<nmodes.fields;i++){
				nmodes.row = mysql_fetch_row(nmodes.res);
				
				do_rawsvsmode(thread,O->nick,nmodes.row[0],_dynadd("-%s",nmodes.row[1]));
				domysql(thread,"UPDATE `orion_core_condata` SET `modes`='' WHERE `nick`='%s'",mysql_quote(thread,nmodes.row[0]));
			}
		}

		mysql_free_result_myr(nmodes.res); //free MySQL result
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O NOOP Perm function for the O NOOP Command.
 * 
 * The O NOOP Perm Command allows an Admin to permanently Noop all IRCOps<br/>
 *   This is useful, if someone hacked an IRCOp-password, to avoid /oper from all IRCOps<br/>
 *   (All /oper requests will result in a /umode -o and a Kill!) - you should change all Oper-Passwords<br/>
 *   Immediately and turn NOOP PERM off again - then you can be sure noone has the new Passwords!
 *  
 * Syntax is: /msg O NOOP PERM { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_noop_perm(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL nmodes;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&nmodes,sizeof(nmodes));

	/*
	 * Noop Perm Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"noop perm")){
		if(streqnc(param[5],"on") && strcor(thread->sql.Cdata.modes,"o")){
			nmodes.res    = doSelectMySQL(thread,"SELECT `nick`,`modes` FROM `orion_core_condata` WHERE `faketype`='0' AND `modes` RLIKE 'o'");
			nmodes.fields = mysql_num_rows(nmodes.res);

			if(m->WallOSNoOp)
				do_global(thread,O->nick,"NoOp PERM was set from %s, Services entered SecureMode (NoOps since unsetting!).",nick);

			noop=1;
			
			if(nmodes.fields){
				for(i=0;i<nmodes.fields;i++){
					nmodes.row = mysql_fetch_row(nmodes.res);
				
					//do_svsmode(thread,snick,nmodes.row[0],"-%s",nmodes.row[1]);
					//domysql(thread,"UPDATE `orion_core_condata` SET `modes`='' WHERE `nick`='%s'",mysql_quote(thread,nmodes.row[0]));
					do_kill(thread,snick,nmodes.row[0],"your olines are nooped.");
				}
			}

			mysql_free_result_myr(nmodes.res); //free MySQL result
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else if(streqnc(param[5],"off")){
			noop=0;
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else send_lang(thread,language,howtosend,snick,nick,"O_ACCESS_DENIED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O NOOP Command (/msg O HELP NOOP)
 * 
 * When the user tries to get help for this command, using /msg O HELP NOOP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP NOOP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_noop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Noop Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help noop")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"noop"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_NOOP");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O NOOP Temp Help function for the O NOOP Command (/msg O HELP NOOP TEMP)
 * 
 * When the user tries to get help for this command, using /msg O HELP NOOP TEMP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP NOOP TEMP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_noop_temp_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Noop Temp Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help noop temp")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"noop temp"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_NOOP_TEMP");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O NOOP Perm Help function for the O NOOPEVER Command (/msg O HELP NOOP PERM)
 * 
 * When the user tries to get help for this command, using /msg O HELP NOOP PERM,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP NOOP PERM
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_noop_perm_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Noop Perm Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help noop perm")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"noop perm"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_NOOP_PERM",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - NOOP part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the NOOP command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_noop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Noop (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_NOOP",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"noop"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
