/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_rnline.c
 *   @brief This is the Module file for O RNLINE Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O RNLINE command.
 *
 *   The O RNLINE Command allows an admin to manage the Realname-Glines.<br/>
 *   (these Glines will be set normal for their realhosts,<br/>
 *    but they are completely processed and controlled by orion!)
 *
 *   Syntax is: /msg O RNLINE { ADD | DEL | LIST } [< OPTION >] [< realname > [:< duration(i.e. 2d) > < reason >]]
 *
 *<table border="0">
 *    <tr><td> ADD	</td><td> Syntax is: /msg O RNLINE ADD	</td><td> { G | K } < realname > :< duration(i.e. 2d) > < reason >	</td></tr>
 *    <tr><td> DEL	</td><td> Syntax is: /msg O RNLINE DEL	</td><td> { G | K } < realname >									</td></tr>
 *    <tr><td> LIST	</td><td> Syntax is: /msg O RNLINE LIST	</td><td> 															</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "O"
#define MODUL "rnline"
#define MODUL_FILE "o_rnline"
#define MODUL_VERSION "1.2"

MOD_BOOL O_rnline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_rnline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_rnline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_rnline_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_rnline_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_rnline_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_rnline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_rnline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"rnline option[1]","rnline helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);
	
	addLeastModuleLevels(thread,&Module,"rnline",870,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */
	
	addCommand(thread,&Module,"rnline add",O_rnline_add,870,INFINIT,6,0);
	addCommand(thread,&Module,"rnline del",O_rnline_del,870,INFINIT,6,0);
	addCommand(thread,&Module,"rnline list",O_rnline_list,870,INFINIT,4,4);
	
	addCommand(thread,&Module,"help rnline",O_rnline_help,870,INFINIT,4,4);
	addCommand(thread,&Module,"help rnline add",O_rnline_add_help,870,INFINIT,5,5);
	addCommand(thread,&Module,"help rnline del",O_rnline_del_help,870,INFINIT,5,5);
	addCommand(thread,&Module,"help rnline list",O_rnline_list_help,870,INFINIT,5,5);
		
	addCommand(thread,&Module,"help",O_rnline_help_listing,870,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O RNLINE Add function for the O RNLINE Command.
 * 
 * The O RNLINE Add Command allows an Admin to add a RealnameGline/RealnameKill for given RealName
 *  
 * Syntax is: /msg O RNLINE ADD { G | K } < realname > :< duration > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_rnline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL condata;
	int change=0, counted=0;
	time_t expiretime=0;
	time_t date	= 0;
	size_t i 	= 0;
	char flag, rngline[500], reason1[500], reason3[500];
	char *reason[ARRAY_SIZE];

	/* Initialise Variables to (NULL) */
	bzero(&condata,sizeof(condata));
	flag = '\0';
	bzero(&rngline,sizeof(rngline));
	bzero(&reason1,sizeof(reason1));
	bzero(&reason3,sizeof(reason3));

	/*
	 * Rnline Add Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rnline add") && param[7]){
		//:shock PRIVMSG O :RNLINE ADD G All EyeZ On Me :20d flooding isn't allowed
		for(i=6;param[i]!=NULL;i++){
			if(param[i][0]==':'){
				change=1;
				straddns(reason1,_dynadd("%s ",param[i]),sizeof(reason1));
			}else{
				if(change==0){
					straddns(rngline,_dynadd("%s ",param[i]),sizeof(rngline));
				}
				else if(change==1){
					straddns(reason1,_dynadd("%s ",param[i]),sizeof(reason1));
				}
			}
		}
		
		if(dynlen(param[5])>1){
			send_lang(thread,language,howtosend,snick,nick,"O_RNLINE_ADD_TO_MANY_FLAGS");
			return MOD_CONT;
		}
		
		substr(reason1,0);
		
		rngline[dynlen(rngline)-1]='\0'; // cut off last ' '
		reason1[dynlen(reason1)-1]='\0'; // cut off last ' '

		bzero(&reason,sizeof(reason));
		strsplit(reason1,reason,' ');

		if(!check_expire(reason[0])){
			send_lang(thread,language,howtosend,snick,nick,"INVALID_EXPIRESTRING",reason[0]);
			return MOD_CONT;
		}

		expiretime=expire(reason[0])+time(NULL);
// 		printf("reason[0] = %s\n",reason[0]);
// 		printf("time = %d\n",expiretime-time(NULL));
		dynsplice(reason,0,0);

		//bzero(&reason3,sizeof(reason3));
		dynncpy(reason3,_djoin(' ',reason),sizeof(reason3));
		date=time(NULL);
		
		if(stricor(param[5],"gk")){
			flag='\0';
			flag=param[5][0];
			counted=0;
				
			// nick!ident@realhost#ip=realname
			// (.*)=realname
				
			if(lower(flag)=='g'){
				domysql(thread,"INSERT INTO `orion_core_glines` (`rng`,`line`,`gline`,`duration`,`addedby`,`reason`) VALUES ('1','G','(.*)=%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE `duration`=VALUES(`duration`),`addedby`=VALUES(`addedby`),`reason`=VALUES(`reason`)",mysql_quote(thread,rngline),expiretime,mysql_quote(thread,nick),mysql_quote(thread,reason3));
			}else if(lower(flag)=='k'){
				domysql(thread,"INSERT INTO `orion_core_glines` (`rng`,`line`,`gline`,`duration`,`addedby`,`reason`) VALUES ('2','G','(.*)=%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE `duration`=VALUES(`duration`),`addedby`=VALUES(`addedby`),`reason`=VALUES(`reason`)",mysql_quote(thread,rngline),expiretime,mysql_quote(thread,nick),mysql_quote(thread,reason3));
			}

			condata.res    = doSelectMySQL(thread,"SELECT CONCAT('*@',`rhost`) AS `realhost`,`nick` FROM `orion_core_condata` WHERE `realname`='%s' AND `faketype`='0'",mysql_quote(thread,rngline));
			condata.fields = mysql_num_rows(condata.res);

			if(condata.fields>0){
				for(i=0;i<condata.fields;i++){
					condata.row = mysql_fetch_row(condata.res);
						
					if(lower(flag)=='g' && condata.row[0]) do_gline(thread,m->servername,condata.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),expiretime,date,reason3);
					if(lower(flag)=='k' && condata.row[1]) do_kill(thread,snick,condata.row[1],reason3);
				}
			}
			mysql_free_result_myr(condata.res); //free MySQL result
			
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else send_lang(thread,language,howtosend,snick,nick,"O_WRONG_PARAMETERS");
	}else{
	/*
	 *  Error Messages
	 */
		if(!param[7])	send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
		else 			send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O RNLINE Del function for the O RNLINE Command.
 * 
 * The O RNLINE Del Command allows an Admin to delete a RealnameGline/RealnameKill for given RealName
 *  
 * Syntax is: /msg O RNLINE DEL { G | K } < realname >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_rnline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL rhost;
	char realname[MAX_REALNAME_LEN];
	char *merged_realname = NULL;
	//size_t 	i = 0;
	//int32 j = 0;

	/* Initialise Variables to (NULL) */
	bzero(&rhost,sizeof(rhost));
	bzero(&realname,sizeof(realname));

	/*
	 * Rnline Del Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rnline del")){
		merged_realname = _dstrjoin(' ',param,6);
		
		if(dynlen(param[5])>1){
			send_lang(thread,language,howtosend,snick,nick,"O_RNLINE_ADD_TO_MANY_FLAGS");
			return MOD_CONT;
		}

		if(dynlen(merged_realname)>sizeof(realname)){
			send_lang(thread,language,howtosend,snick,nick,"MASK_TOO_LONG",merged_realname,sizeof(realname));
			return MOD_CONT;
		}
		
		dynncpy(realname,_dstrjoin(' ',param,6),sizeof(realname));

		if(lower(param[5][0])=='g'){
			domysql(thread,"DELETE FROM `orion_core_glines` WHERE `gline`='(.*)=%s' AND `rng`='1'",mysql_quote(thread,realname));

			/*rhost.res    = doSelectMySQL(thread,"SELECT `rhost` FROM `orion_core_condata` WHERE `realname`='%s'",mysql_quote(thread,realname));
			rhost.fields = mysql_num_rows(rhost.res);

			if(rhost.fields>0){
				for(j=0;j<rhost.fields;j++){
					rhost.row = mysql_fetch_row(rhost.res);
						
					do_ungline(thread,m->servername,rhost.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),time(NULL));
				}
			}
			mysql_free_result_myr(rhost.res);*/

			send_lang(thread,language,howtosend,snick,nick,"O_RNLINE_DEL_REMOVED_GLINE_FROM_REALNAMEGLINES",realname);
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else if(lower(param[5][0])=='k'){
			domysql(thread,"DELETE FROM `orion_core_glines` WHERE `gline`='(.*)=%s' AND `rng`='2'",mysql_quote(thread,realname));
			send_lang(thread,language,howtosend,snick,nick,"O_RNLINE_DEL_REMOVED_GLINE_FROM_REALNAMEGLINES",realname);
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O RNLINE List function for the O RNLINE Command.
 * 
 * The O RNLINE List Command allows an Admin to dump the list of RealnameGlines/RealnameKills
 *  
 * Syntax is: /msg O RNLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_rnline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
    deepSQL MySQL;
	int i=0;
	char time_return[150];
	time_t gettime;
	
	/* Initialise Variables to (NULL) */
	bzero(&MySQL,sizeof(MySQL));
	bzero(&time_return,sizeof(time_return));
	
	/*
	 * Rnline List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rnline list")){
		MySQL.res    = doSelectMySQL(thread,"SELECT `gline`,`duration`,`addedby`,`reason`,`rng` FROM `orion_core_glines` WHERE `rng` IN (1,2) ORDER BY `rng` ASC");
		MySQL.fields = mysql_num_rows(MySQL.res);
		
		if(MySQL.fields>0){
			for(i=0;i<MySQL.fields;i++){
				MySQL.row = mysql_fetch_row(MySQL.res);
				
				if(stoi(MySQL.row[1])==0) bzero(&time_return,sizeof(time_return));
				else{
					gettime=(time_t)stol(MySQL.row[1]);
					//bzero(&time_return,sizeof(time_return));
					dynncpy(time_return,get_localtime(gettime),sizeof(time_return));
				}
				
				int realname_position = getposition(MySQL.row[0],'=');
				realname_position++;
			
				if(i==0) 						send_lang(thread,language,howtosend,snick,nick,"O_RNLINE_LIST_HEADER_TIME_NOW",get_localtime(time(NULL)));
				if(stoi(MySQL.row[4])==1) 		send_lang(thread,language,howtosend,snick,nick,"O_RNLINE_LIST_LIST_RG_LINES",i+1,&MySQL.row[0][realname_position],MySQL.row[3],MySQL.row[2],time_return);
				else if(stoi(MySQL.row[4])==2)	send_lang(thread,language,howtosend,snick,nick,"O_RNLINE_LIST_LIST_RK_LINES",i+1,&MySQL.row[0][realname_position],MySQL.row[3],MySQL.row[2],time_return);
			}
		}
		
        mysql_free_result_myr(MySQL.res); //free MySQL result
		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O RNLINE Command (/msg O HELP RNLINE)
 * 
 * When the user tries to get help for this command, using /msg O HELP RNLINE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP RNLINE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_rnline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Rnline Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help rnline")){
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_RNLINE",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O RNLINE Add Help function for the O RNLINE Command (/msg O HELP RNLINE ADD)
 * 
 * When the user tries to get help for this command, using /msg O HELP RNLINE ADD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP RNLINE ADD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_rnline_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Rnline Add Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help rnline add")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rnline add"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_RNLINE_ADD");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O RNLINE Del Help function for the O RNLINE Command (/msg O HELP RNLINE DEL)
 * 
 * When the user tries to get help for this command, using /msg O HELP RNLINE DEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP RNLINE DEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_rnline_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Rnline Del Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help rnline del")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rnline del"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_RNLINE_DEL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O RNLINE List Help function for the O RNLINE Command (/msg O HELP RNLINE LIST)
 * 
 * When the user tries to get help for this command, using /msg O HELP RNLINE LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP RNLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_rnline_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Rnline List Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help rnline list")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rnline list"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_RNLINE_LIST");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - RNLINE part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the RNLINE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_rnline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Rnline (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_RNLINE",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rnline"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
