/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_bandel.c
 *   @brief This is the Module file for Q BANDEL Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q BANDEL command.
 *
 *   The Q BANDEL Command allows the user to delete<br/>
 *   a permanent host ban from a channel he has enough access to.
 *
 *   Syntax is: /msg Q BANDEL #channel < ban >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "Q"
#define MODUL "bandel"
#define MODUL_FILE "q_bandel"
#define MODUL_VERSION "1.2"

MOD_BOOL Q_bandel(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_bandel_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_bandel_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"bandel","bandel");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"bandel",Q_bandel,450,1,5,5);
	addCommand(thread,&Module,"help bandel",Q_bandel_help,100,1,4,4);
	addCommand(thread,&Module,"showcommands",Q_bandel_showcommands,100,1,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q BANDEL Command.
 * 
 * The Q BANDEL Command allows the user to delete<br/>
 *   a permanent host ban from a channel he has enough access to.
 *
 * Syntax is: /msg Q BANDEL #channel < ban >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_bandel(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	char CBot[MAX_NICK_LEN];
	
	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Bandel Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"bandel")){
		/* Updates the Threads Channel-Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set Q to our Bot, if BotServ is not loaded, or no Bot is assigned, or if SYMBIOSIS is OFF */
		QCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));

		if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
			if((thread->c.chan.settings & CS_FORBID_BIT))		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
			else if((thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
			else if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"bandel") || strcor(thread->c.chan.access.ownlevel,"nm")){
				if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_cs_akick` WHERE `channel`='%s' AND `mask`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5])))==1){
					do_mode(thread,CBot,param[4],thread->c.chan.datecreate,"-b %s",param[5]);
					domysql(thread,"DELETE FROM `orion_cs_akick` WHERE `channel`='%s' AND `mask`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5]));
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"Q_BANDEL_NO_SUCH_BUN",param[5],param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"Q_NO_N_OR_M_FLAG");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q BANDEL Command (/msg Q HELP BANDEL)
 * 
 * When the user tries to get help for this command, using /msg Q HELP BANDEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP BANDEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_bandel_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Bandel Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help bandel")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_BANDEL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - BANDEL part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the BANDEL command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_bandel_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Bandel (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_BANDEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"bandel"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
