/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file topic.c
 *   @brief This is the Module file for TOPIC (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the TOPIC command.
 *
 *   The TOPIC File is used to process TOPIC changed sent from Server
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Topic is used to set Topics or receive Topics - and managing the DB in this case.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"
#include "webform.h"

/**
 * @brief do_topic:  Sends a TOPIC (depending on the Loaded Protocol) - Using TOPIC
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick which should set the Topic (ServicesNick ONLY)
 * @param last_setter The Nick who set the Topic (the last setter)
 * @param channel The Channel in which we want to set the Topic
 * @param date The Channels creation date
 * @param ftopic The new Topic
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_topic(Threads *thread, char *nick, char *last_setter, char *channel, time_t date, const char *ftopic, ...){
	PROTOCOL_BOOL analyzer;
	
	/* Let'z check out inputs - are they valid? */	
	if(nick==NULL || channel==NULL || ftopic==NULL)
		return CMD_ERROR;
		
	/* Deklarate Variables */
	va_list va;
	char topic[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&topic,sizeof(topic));

	/* Join reason string */
	va_start	(va,  ftopic);
	vsnprintf	(topic, sizeof(topic)-1, ftopic, va);
	va_end		(va);
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,channel,NULL);

	if(topic[0]!='\0'){
		printf("setting new topic\n");

		/* Analyze Command - is it supported from the IRCd !? */
		analyzer = orion_WRCtranslate_cmd_topic(thread, nick, channel, last_setter, topic, date);
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;
		
		/* 	Update our Cache	*/
		bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));
		bzero(&thread->c.chan.last_topic_setter,sizeof(thread->c.chan.last_topic_setter));
		
		dynncpy(thread->c.chan.last_topic,topic,sizeof(thread->c.chan.last_topic));
		dynncpy(thread->c.chan.last_topic_setter,last_setter,sizeof(thread->c.chan.last_topic_setter));
		thread->c.chan.last_topic_time = time(NULL);
		/**			***			**/
		
		domysql(thread,"UPDATE `orion_cs_core` SET `last_topic`='%s', `last_topic_setter`='%s', `last_topic_time`='%ld' WHERE `channel`='%s'",mysql_quote(thread,topic),mysql_quote(thread,last_setter),thread->c.chan.last_topic_time,mysql_quote(thread,channel));
	}
	else {
		printf("do_topic(); wanted to set a new topic - but it was empty =(\n");
		return CMD_ERROR;
	}
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief topic:  Receives a TOPIC ON SERVERLINK (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick which set the Topic
 * @param channel The Channel in which the TOPIC was set
 * @param date The Channels creation date
 * @param topic The new Topic
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL topic(Threads *thread, char *nick, char *channel, time_t date, char* topic){
//TOPIC on SERVERLINK (after mode handling)
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,channel,NULL);

	if(thread->c.chan.fields && thread->c.chan.last_topic[0]!='\0' && strcor("t",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q"))
		//do_topic(thread,Q->nick,thread->c.chan.name,thread->c.chan.datecreate,thread->c.chan.last_topic); // - Changed 29.04.2008 (don't know if it's working), should avoid double topic changing
		printf("old do topic for Q\n");
	else if(thread->c.chan.fields && thread->c.chan.last_topic[0]!='\0' && (thread->c.chan.settings&CS_KEEPTOPIC_BIT) && streqnc(thread->c.chan.serv,"chanserv"))
		//do_topic(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,thread->c.chan.last_topic); // - Changed 29.04.2008 (don't know if it's working), should avoid double topic changing
		printf("old do topic for ChanServ\n");
	else{
		printf("topic(); we are now updating to a new topic\n");
		/* 	Update our Cache	*/
		bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));
		bzero(&thread->c.chan.last_topic_setter,sizeof(thread->c.chan.last_topic_setter));
		
		dynncpy(thread->c.chan.last_topic,topic,sizeof(thread->c.chan.last_topic));
		dynncpy(thread->c.chan.last_topic_setter,nick,sizeof(thread->c.chan.last_topic_setter));
		thread->c.chan.last_topic_time = date;
		/**			***			**/
		
		domysql(thread,"UPDATE `orion_cs_core` SET `last_topic`='%s', `last_topic_setter`='%s', `last_topic_time`='%ld' WHERE `channel`='%s'",mysql_quote(thread,topic),mysql_quote(thread,nick),date,mysql_quote(thread,thread->c.chan.name));
	}
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief topic:  Receives a TOPIC /topic (user request) (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick which set the Topic
 * @param channel The Channel in which the TOPIC was set
 * @param date The Channels creation date
 * @param ftopic The new Topic
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_topic(Threads *thread, char *nick, char *channel, time_t date, char* ftopic){
//TOPIC on SET (request by a User)
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,channel,NULL);
	char odate[50];
	char *eventarray[5];
	
	if(thread->c.chan.fields && thread->c.chan.serv[0]=='\0'){
		/* 	Update our Cache	*/
		bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));
		bzero(&thread->c.chan.last_topic_setter,sizeof(thread->c.chan.last_topic_setter));
		
		dynncpy(thread->c.chan.last_topic,ftopic,sizeof(thread->c.chan.last_topic));
		dynncpy(thread->c.chan.last_topic_setter,nick,sizeof(thread->c.chan.last_topic_setter));
		thread->c.chan.last_topic_time = date;
		/**			***			**/
		
		domysql(thread,"UPDATE `orion_cs_core` SET `last_topic`='%s', `last_topic_setter`='%s', `last_topic_time`='%ld' WHERE `channel`='%s'",mysql_quote(thread,ftopic),mysql_quote(thread,nick),date,mysql_quote(thread,thread->c.chan.name));
		return CMD_EXECUTED;
	}
	else{
		bzero(eventarray,sizeof(eventarray));
		bzero(&odate,sizeof(odate));
		sprintf(odate,"%ld",date);
		
		eventarray[0] = nick;
		eventarray[1] = channel;
		eventarray[2] = odate;
		eventarray[3] = ftopic;
		eventarray[4] = NULL;
		/************* SEND on_topic to the eventhandler ****************/
		_ExecuteEvent(nick,eventarray,"on_topic");
		/****************************************************************/
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */
